<?php
// +----------------------------------------------------------------------+
// | sa auth module - user management xmlrpc server                       |
// +----------------------------------------------------------------------+
// | Copyright (c) 2002 serie a - Digitale Medien und Systeme GmbH        |
// +----------------------------------------------------------------------+
// | Authors: Jrn Allert <allert@serie-a.de>                             |
// |          Rene Renk <renk@serie-a.de>                                 |
// +----------------------------------------------------------------------+
//
// $Id: prob.tree.inc.php,v 1.1.1.1 2004/07/15 13:30:24 opensurveypilot Exp $

// start session
@session_start();

// require all configuration files and libraries
require_once("../config/config.inc.php"); // common config file
require_once($cfgPathToConf."/file.inc.php"); // project files
require_once($cfgPathToAuthFiles."/config/auth.file.inc.php"); // auth files
require_once($cfgPathToSurveyFiles."/config/survey.config.inc.php"); // survey configuration
require_once($cfgPathToSurveyFiles."/config/survey.file.inc.php"); // survey files
require_once($cfgPathToStatisticFiles."/config/statistic.file.inc.php"); // statistic files
require_once($cfgPathToAuthFiles."/lib/permission.inc.php"); // permission functions
require_once ("DB.php"); // PEAR::DB

/**
*	Class to create Javascript which in turn creates the group tree	
*
*	@author otto <otto@poellath.org>
*	@see PEAR::DB
*/
class GroupTreeMenu
{
	/**
	* Name of the image used as node icon
	* @var 		string $icon
	* @access 	private
	*/
    var $icon = "folder.gif";

	/**
	* The menu object
	* @var 		object HTML_TreeMenu $menu
	* @access 	private
	*/
    var $menu;


	/**
	* The DB handle
	* @var 		mixed $db
	* @see		DB::connect()
	* @access 	private
	*/
	var $db = "";

	/**
	* Name the DB table node elements are read from
	* @var 		string $table
	* @access 	private
	*/
    var $table;
    
	/**
	* The time when tree building is started
	* @var 		int $startTime
	* @access 	private
	*/
    var $startTime;

	/**
	* The time in seconds after which tree building is aborted
	* @var 		int $timeout
	* @access 	private
	*/
    var $timeout = 10;
	
	/**
	* Name of the image used as design node icon
	* @var 		string $designIcon
	* @access 	private
	*/
    var $designIcon = "designfolder.gif";
    
    /**
	* Name of the image used as security node icon
	* @var 		string $securityIcon
	* @access 	private
	*/
    var $rawIcon = "securityfolder.gif";

	/**
	* The last error message
	* @var 		string $error;
	* @access 	private
	*/
    var $error;
    
	/**
	*	Constructor
	*
	*	@access public
	*	@param string $table 	Name the DB table node elements are read from
	*/
    function GroupTreeMenu()
    {
        $this->db= DB::connect(DSN, true);
        if (DB::isError($this->db)) 
        {    
           return false;
        }
        
        $this->buildTree();
    }
    
	/**
	*	Check for an error since last invocation of buildTree()
	*
	*	@access public
	*	@return boolean     TRUE if a timeout occurred, FALSE otherwise
	*/
    function isError()
    {
    	return ($this->error != "");
    }
    
	/**
	*	Get the error message
	*
	*	@access public
	*	@return string  Error message
	*/
    function getError()
    {
    	return $this->error;
    }
	
	
	function appendDesignNode(&$node, $nodeId)
    {
	global $lang_script;
	
	//todo: "Design" as instance var
		$node->addItem(new HTML_TreeNode($lang_script["prob.tree.inc.php"]["nodeFormMask"], $this->getLinkForDesignId($nodeId), $this->designIcon)); //todo: instance var
	}
    
    function appendRawNode(&$node, $nodeId)
    {
    	global $lang_script;
	
	$node->addItem(new HTML_TreeNode($lang_script["prob.tree.inc.php"]["nodeRawDataEdit"], $this->getLinkForRawId($nodeId), $this->rawIcon)); //todo: instance var
	}
	
	function appendUserNode(&$node, $nodeId)
    {
    	global $lang_script;
	
	$node->addItem(new HTML_TreeNode($lang_script["prob.tree.inc.php"]["nodePanelUserGroup"], $this->getLinkForUserId($nodeId), "detailfolder.gif")); //todo: instance var
	}
    
    
	/**
	*	Build the user tree
	*
	*	@access private
	*/
    function buildTree()
    {
	global $lang_script;
	
    	$this->error = "";
        $this->startTime = time();
        $this->menu = new HTML_TreeMenu("menuLayer", '../media/images/treemenu', '_self');
        
    	$rootNode = new HTML_TreeNode($lang_script["prob.tree.inc.php"]["rootnodePanelUserOverview"], $this->getLinkForRootNode(), "rootfolder.gif");
		$this->menu->addItem($rootNode);
    	// add design node
		$this->appendDesignNode($rootNode, 1);
        // add rawdata node
        $this->appendRawNode($rootNode, 1);
		// add user node
		$this->appendUserNode($rootNode, 1);
		
		//$this->appendChildrenToNode($rootNode, 0);
    }
    
	/**
	*	Append all existing child nodes to a node
	*
	*	@access private
	*	@param object HTML_TreeNode &$node		Node to append children to
	*	@param string $nodeId					The node's DB-ID
	*/
    function appendChildrenToNode(&$node, $nodeId)
    {
    	if(time() - $this->startTime > $this->timeout)
        { 
			$this->error = "Timeout. Tree build aborted after ".(time() - $this->startTime)." seconds.";
    		return false;
    	}
    
    	$children = $this->getChildrenForNodeWithId($nodeId);
    	if($children)
        {
			for($i=0; $i<count($children); $i++)
            {
				$node->addItem($children[$i][1]);
				$this->appendChildrenToNode($children[$i][1], $children[$i][0]);
			}
    	}
    }
    
	/**
	*	Holt alle Kinder eines Knotens aus der DB
	*
	*	@access private
	*	@param string $id						The node's DB ID
	*	@return mixed							FALSE if no children are found, otherwise an array with 1 element for each child found,
    * 											each consisting of an array containing the child node's DB ID and an object of type HTML_TreeNode
	* 											representing the child node.
	*/
    function getChildrenForNodeWithId($id)
    {
    	global $cfgTabAuthGroup;
		
		$children = false;
    	
        $sql = "select GROUP_ID_PK, GROUP_NAME 
			from $cfgTabAuthGroup 
			where GROUP_PARENT_ID = '$id' 
			order by GROUP_NAME";
		//
        $result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
            return "Fehler. ".$this->db->getMessage();
        }
        
        if ($result->numRows() == 0)
        {
            return false;
        }
        
        // fetch userdata
        while ($row = $result->fetchRow(DB_FETCHMODE_ASSOC)) 
        {
            $children[] = array($row['GROUP_ID_PK'], new HTML_TreeNode($row['GROUP_NAME'], $this->getLinkForId($row['GROUP_ID_PK']), $this->icon));
        }
    	
    	return $children;
    }
    
	/**
	*	Creates the URL to link a node to
	*
	*	@access private
	*	@param string $id						The node's DB ID
	*/
    function getLinkForId($id)
    {
    	global $fileUserSubfolderToplevel, $fileFrameToplevelName;
        global $fileUserOverview, $fileFrameContentName;
        
    	return "javascript:changeTwoFrames(\'$fileUserSubfolderToplevel?id=$id\',\'$fileFrameToplevelName\',\'$fileUserOverview?id=$id\',\'$fileFrameContentName\')";
    }

    function getLinkForRootNode()
    {
		global $fileProbCommonToplevel, $fileFrameToplevelName;
    	global $fileProbOverview, $fileFrameContentName;

    	return "javascript:changeTwoFrames(\'$fileProbCommonToplevel\',\'$fileFrameToplevelName\',\'$fileProbOverview\',\'$fileFrameContentName\')";
    }
	
	/**
	*	Echos the Javascript code to create the user tree
	*
	*	@access public
	*/
	function printMenu()
    {
		$this->menu->printMenu();
	}
	
	/**
	*	Creates the URL for design node
	*
	*	@access private
	*	@param string $id   The node's DB ID
	*/
    function getLinkForDesignId($id)
    {
    	global $fileProbCommonToplevel, $fileFrameToplevelName;
    	global $fileProbMaskOverview, $fileFrameContentName;

    	return "javascript:changeTwoFrames(\'$fileProbCommonToplevel?mask=1\',\'$fileFrameToplevelName\',\'$fileProbMaskOverview\',\'$fileFrameContentName\')";
    }
	
	/**
	*	Creates the URL for user node
	*
	*	@access private
	*	@param string $id   The node's DB ID
	*/
    function getLinkForUserId($id)
    {
    	global $fileProbCommonToplevel, $fileFrameToplevelName;
    	global $fileProbGroupOverview, $fileFrameContentName;

    	return "javascript:changeTwoFrames(\'$fileProbCommonToplevel?group=1\',\'$fileFrameToplevelName\',\'$fileProbGroupOverview\',\'$fileFrameContentName\')";
    }
    
    /**
	*	Creates the URL for rawdata node
	*
	*	@access private
	*	@param string $id   The node's DB ID
	*/
    function getLinkForRawId($id)
    {
    	global $fileProbCommonToplevel, $fileFrameToplevelName;
    	global $fileProbOverview, $fileFrameContentName;

    	return "javascript:changeTwoFrames(\'$fileProbCommonToplevel?import=1\',\'$fileFrameToplevelName\',\'$fileProbOverview?raw=1\',\'$fileFrameContentName\')";
    }
    
}   // end of class GroupTreeMenu

?>
