<?php
// +----------------------------------------------------------------------+
// | sa auth module - user management xmlrpc server                       |
// +----------------------------------------------------------------------+
// | Copyright (c) 2002 serie a - Digitale Medien und Systeme GmbH        |
// +----------------------------------------------------------------------+
// | Authors: Jrn Allert <allert@serie-a.de>                             |
// |          Rene Renk <renk@serie-a.de>                                 |
// +----------------------------------------------------------------------+
//
// $Id: statistics.tree.inc.php,v 1.1.1.1 2004/07/15 13:31:26 opensurveypilot Exp $

// start session
@session_start();

// require all configuration files and libraries
require_once($cfgPathToProjectAdmin."/config/config.inc.php"); // common config file // common config file
require_once($cfgPathToConf."/file.inc.php"); // project files
require_once($cfgPathToAuthFiles."/config/auth.file.inc.php"); // auth files
require_once($cfgPathToSurveyFiles."/config/survey.config.inc.php"); // survey configuration
require_once($cfgPathToSurveyFiles."/config/survey.file.inc.php"); // survey files
require_once($cfgPathToStatisticFiles."/config/statistic.file.inc.php"); // statistic files
require_once($cfgPathToAuthFiles."/lib/permission.inc.php"); // permission functions
require_once ("DB.php"); // PEAR::DB

/**
*	Class to create Javascript which in turn creates the project tree (which is not really a tree)
*
*	@author otto <otto@poellath.org>
*	@see PEAR::DB
*/
class StatTreeMenu
{
	/**
	* Name of the image used as node icon
	* @var 		string $icon
	* @access 	private
	*/
    var $icon = "folder.gif";
    
    /**
	* Name of the image used as node icon
	* @var 		string $icon
	* @access 	private
	*/
    var $iconSubFolderStart = "subfolderstart.gif";
    
    /**
	* Name of the image used as node icon
	* @var 		string $icon
	* @access 	private
	*/
    var $iconSubFolderEnd = "subfolderend.gif";
    
    /**
	* Name of the image used as node icon
	* @var 		string $icon
	* @access 	private
	*/
    var $iconSubFolderEndReturn = "subfolderendreturn.gif";
    
    /**
	* Name of the image used as root node icon
	* @var 		string $rootIcon
	* @access 	private
	*/
    var $rootIcon = "rootfolder.gif";

	/**
	* ID of the auhor
	* @var 		int $authorId
	* @access 	private
	*/
    var $authorId;

	/**
	* The menu object
	* @var 		object HTML_TreeMenu $menu
	* @access 	private
	*/
    var $menu;

	/**
	* The DB handle
	* @var 		mixed $db
	* @see		DB::connect()
	* @access 	private
	*/
	var $db = "";

	/**
	* Name the DB table node elements are read from
	* @var 		string $table
	* @access 	private
	*/
    var $table;
    
	/**
	* The time when tree building is started
	* @var 		int $startTime
	* @access 	private
	*/
    var $startTime;

	/**
	* The time in seconds after which tree building is aborted
	* @var 		int $timeout
	* @access 	private
	*/
    var $timeout = 10;

	/**
	* The last error message
	* @var 		string $error;
	* @access 	private
	*/
    var $error;
   
	/**
	*   Constructor
	*
	*   @access public
	*   @param string $table 	    Name the DB table node elements are read from
    *   @param id     $authorId 	id of author
	*/
    function StatTreeMenu($authorId)
    {
        $this->authorId = $authorId;
        $this->selectedProjectId = $selectedProjectId;
        
        $this->db= DB::connect(DSN, true);
        if (DB::isError($this->db)) 
        {    
           return false;
        }
        
    	$this->error = "";
        $this->startTime = time();
        $this->menu = new HTML_TreeMenu("menuLayer", '../media/images/treemenu', '_self');

        $this->buildTree();
    }
    
	/**
	*	Check for an error since last invocation of buildTree()
	*
	*	@access public
	*	@return boolean     TRUE if a timeout occurred, FALSE otherwise
	*/
    function isError()
    {
    	return ($this->error != "");
    }
    
	/**
    *	Get the error message
	*
	*	@access public
	*	@return string      Error message
	*/
    function getError()
    {
    	return $this->error;
    }
    
	/**
	*	Build the project tree
	*
	*	@access private
	*/
    function buildTree()
    {
	global $lang_script;
        // root node
    	$rootNode = new HTML_TreeNode($lang_script["statistic.tree.inc.php"]["rootnodeProjectOverview"], $this->getLinkForRootNode(), "rootfolder.gif");
    	$this->menu->addItem($rootNode);
    	
        // get all projects
    	$projects = $this->getProjects();
        
        // create new permissionFunctions object
        $permission = &new permissionFunctions;
        
    	if($projects)
        {
			for($i=0; $i<count($projects); $i++)
            {
				// check permission
                $permissionMode = $permission->getPermissionMode($_SESSION["sessionUserId"], $projects[$i][0], 1);
                // permission must be bigger than 0.
                if($permissionMode >= 1)
                {
				    $rootNode->addItem($projects[$i][1]);
                    // add group node
				   // $this->appendGroupNodeToProject($projects[$i][1], $projects[$i][0]);
                }
			}
    	}
    }
    	
    function appendGroupNodeToProject(&$node, $nodeId)
    {
	global $lang_script;
    	$groupNode = new HTML_TreeNode($lang_script["statistic.tree.inc.php"]["groupNodeQuestiongroups"], $this->getLinkForAllGroupId($nodeId), $this->icon);
		$this->appendGroupsToGroupNode($groupNode, $nodeId, 0);
		$node->addItem(&$groupNode);
	}
	
    function appendGroupsToGroupNode(&$node, $nodeId, $parentId)
    {
    
        $children = $this->getChildrenForNodeWithId($nodeId, $parentId);
        if($children)
        {
			for($i=0; $i<count($children); $i++)
            {
				$node->addItem($children[$i][1]);
                $this->appendGroupsToGroupNode($children[$i][1], $children[$i][0], $children[$i][2] );
			}
    	}
    }
    
    /**
	*	Holt alle Kinder eines Knotens aus der DB
	*
	*	@access private
	*	@param string $id						The node's DB ID
	*	@return mixed							FALSE if no children are found, otherwise an array with 1 element for each child found,
	* 											each consisting of an array containing the child node's DB ID and an object of type HTML_TreeNode
	* 											representing the child node.
	*/
    function getChildrenForNodeWithId($nodeId, $parentId)
    {
        global $cfgSrvyTabForms;
        
    	$children = false;
    	
        if($parentId == 0)
        {
            $orderBy = "FORM_ORDER";
        }
        else
        {
            $orderBy = "FORM_NAME";
        }
        $sql = "select FORM_ID_PK, FORM_PROJECT_ID_FK, FORM_NAME 
			from $cfgSrvyTabForms
			where FORM_PROJECT_ID_FK = $nodeId 
			and FORM_PARENT_ID = $parentId 
			and FORM_NODEGROUP = 0 
			order by $orderBy";
        //
        $result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
            return "Fehler. ".$this->db->getMessage();
        }
        if ($result->numRows() == 0)
        {
            return false;
        }
        // get all nodes
        while ($row = $result->fetchRow(DB_FETCHMODE_ASSOC)) 
        {
            $startNodeArray[] = $row;
        }
        
        for ($i=0;$i<count($startNodeArray);$i++)
		{
            if($parentId == "0")
            {
                $children[] = array($nodeId, new HTML_TreeNode($startNodeArray[$i]["FORM_NAME"], $this->getLinkForGroupId($startNodeArray[$i]["FORM_PROJECT_ID_FK"], $startNodeArray[$i]["FORM_ID_PK"]), $this->icon), $startNodeArray[$i]["FORM_ID_PK"]);
            }
            else
            {
                $groupId = $startNodeArray[$i]["FORM_ID_PK"];
                // check for more folder in this subgroup
                $sql = "select FORM_ID_PK, FORM_PROJECT_ID_FK, FORM_NAME 
					from $cfgSrvyTabForms
					where FORM_PROJECT_ID_FK = $nodeId 
					and FORM_PARENT_ID = $parentId 
					and FORM_NODEGROUP = $groupId 
					order by FORM_ORDER";
                //
                $result = $this->db->query($sql);
                if (DB::isError($result)) 
                {
                    echo "Fehler. ".$this->db->getMessage();
                }
                unset($nodeArray);
                while ($row = $result->fetchRow(DB_FETCHMODE_ASSOC)) 
                {
                    $nodeArray[] = $row;
                }
                $num = count($nodeArray);
                
                if($num == "0")
                {
                    $children[] = array($nodeId, new HTML_TreeNode($startNodeArray[$i]["FORM_NAME"], $this->getLinkForGroupId($startNodeArray[$i]["FORM_PROJECT_ID_FK"], $startNodeArray[$i]["FORM_ID_PK"]), $this->iconSubFolderEnd), $startNodeArray[$i]["FORM_ID_PK"]);
                }
                else
                {
                    $children[] = array($nodeId, new HTML_TreeNode($startNodeArray[$i]["FORM_NAME"], $this->getLinkForGroupId($startNodeArray[$i]["FORM_PROJECT_ID_FK"], $startNodeArray[$i]["FORM_ID_PK"]), $this->iconSubFolderStart), $startNodeArray[$i]["FORM_ID_PK"]);
                }
                
                for ($l=0;$l<$num;$l++)
                {
                    if(($l+1) < $num)
                    {  
                        $children[] = array($nodeId, new HTML_TreeNode($nodeArray[$l]["FORM_NAME"], $this->getLinkForGroupId($nodeArray[$l]["FORM_PROJECT_ID_FK"], $nodeArray[$l]["FORM_ID_PK"]), $this->icon), $nodeArray[$l]["FORM_ID_PK"]);
                    }
                    else
                    {  
                        $children[] = array($nodeId, new HTML_TreeNode($nodeArray[$l]["FORM_NAME"], $this->getLinkForGroupId($nodeArray[$l]["FORM_PROJECT_ID_FK"], $nodeArray[$l]["FORM_ID_PK"]), $this->iconSubFolderEnd), $nodeArray[$l]["FORM_ID_PK"]);
                    }
                }
		    }
        }
        
        return $children;
    }
    
    function getProjects()
    {
    	global $cfgSrvyTabSurvey;
		
		$projects = false;
		
        $sql = "select PROJECT_ID_PK, PROJECT_NAME 
			from $cfgSrvyTabSurvey 
			order by PROJECT_NAME";
        //
        $result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
            return "Fehler. ".$this->db->getMessage();
        }
        if ($result->numRows() == 0)
        {
            return false;
        }
        
        // fetch userdata
        while ($row = $result->fetchRow()) 
        {
            $projects[] = array($row[0], new HTML_TreeNode($row[1], $this->getLinkForProjectId($row[0]), $this->icon));
        }
        
       	return $projects;
    }
    
	/**
	*	Creates the URL to link a node to
	*
	*	@access private
	*	@param string $id       The node's DB ID
	*/
    function getLinkForProjectId($id)
    {
    	global $fileStatisticToplevel, $fileFrameToplevelName;
    	global $fileStatisticOverview, $fileFrameContentName;
		
    	return "javascript:changeTwoFrames(\'$fileStatisticToplevel?projectSelectedPID=$id\',\'$fileFrameToplevelName\',\'$fileStatisticOverview?projectId=$id\',\'$fileFrameContentName\')";
    }

	/**
	*	Creates the URL to link a node to
	*
	*	@access private
	*	@param string $id       The node's DB ID
	*/
    function getLinkForAllGroupId($id)
    {
    	global $fileStatisticToplevel, $fileFrameToplevelName;
    	global $fileStatisticGroupOverview, $fileFrameContentName;
		
    	return "javascript:changeTwoFrames(\'$fileStatisticToplevel?allGroupsSelectedPid=$id\',\'$fileFrameToplevelName\',\'$fileStatisticGroupOverview?projectId=$id\',\'$fileFrameContentName\')";
    }
    
    /**
	*	Creates the URL to link a node to
	*
	*	@access private
	*	@param string $id   The node's DB ID
	*/
    function getLinkForGroupId($projectId, $groupId)
    {
    	global $fileStatisticToplevel, $fileFrameToplevelName;
    	global $fileStatisticGroupOverview, $fileFrameContentName;
        
        return "javascript:changeTwoFrames(\'$fileStatisticToplevel?projectId=$projectId&oneGroupSelectedGid=$groupId\',\'$fileFrameToplevelName\',\'$fileStatisticGroupOverview?projectId=$projectId&groupId=$groupId\',\'$fileFrameContentName\')";
    }
	    
    function getLinkForRootNode()
    {
    	global $fileStatisticToplevel, $fileFrameToplevelName;
    	global $fileStatisticContent, $fileFrameContentName;

    	return "javascript:changeTwoFrames(\'$fileStatisticToplevel\',\'$fileFrameToplevelName\',\'$fileStatisticContent\',\'$fileFrameContentName\')";
    }

	/**
	*	Echos the Javascript code to create the user tree
	*
	*	@access public
	*/
	function printMenu()
    {
		$this->menu->printMenu();
	}
    
} // end of class StatTreeMenu
?>
