<?php
// +----------------------------------------------------------------------+
// | survey pilot                                                         |
// +----------------------------------------------------------------------+
// | Copyright (c) 2002 serie a - Digitale Medien und Systeme GmbH        |
// +----------------------------------------------------------------------+
// | Authors: Jrn Allert <allert@serie-a.de>                             |
// |          Rene Renk <renk@serie-a.de>                                 |
// +----------------------------------------------------------------------+
// | Description:                                                         |
// +----------------------------------------------------------------------+
//
// $Id: survey.inc.php,v 1.3 2007/01/03 10:43:14 easy_israel Exp $

// require all configuration files and libraries
require_once($cfgPathToProjectAdmin."/config/config.inc.php"); // common config file // common config file
require_once($cfgPathToConf."/file.inc.php"); // project files
require_once($cfgPathToSurveyFiles."/config/survey.config.inc.php"); // auth config file
require_once($cfgPathToAuthFiles."/lib/permission.inc.php"); // permission functions
require_once ("DB.php"); // PEAR::DB

/**
* surveyFunctions class
*
* @author  Jrn Allert <allert@serie-a.de>
* @author  Rene Renk <renk@serie-a.de>
* @access  public
*/

class surveyFunctions
{
    /**
    *	handle for db connection
    *	@var    string  $db
    */
    var $db = "";
    
    /**
    *	error message
    *	@var    string  $errorMessage
    */
    var $errorMessage;
    
    /**
    *	constructor 
    *
    *	@access	private
    */
    function surveyFunctions()
    {
        $this->errorMessage = "";
        // connect to db
        $this->db = DB::connect(DSN, true);
        if (DB::isError($this->db)) 
        {    
           $this->setErrorMessage($db->getMessage());
        }
    }
    
    /**
    *	sets error message
    *
    *	@access	private
    *	@param	string	$err
    */
    function setErrorMessage($err)
    {
        $this->errorMessage = $err;
    }
    
    /**
    *	adds a survey
    *
    *	@access	public
    *	@param	array	$surveyArray
    *   @return int $nextId id of the new survey
    */
    function addSurvey($surveyArray)
    {
        global $cfgTabSurvey;
        
        // extract submitted values
		$surveyName 			= $surveyArray["surveyName"];
		$surveyAuthor 			= $surveyArray["surveyAuthor"];
		$surveyDescription 		= $surveyArray["surveyDescription"];
		$surveyOutputdir 		= $surveyArray["surveyOutputDir"];
		$surveyActive 			= $surveyArray["surveyActive"];
		$surveyStartdate 		= $surveyArray["surveyStartDate"];
		$surveyEnddate 			= $surveyArray["surveyEndDate"];
		$surveyLang 			= $surveyArray["surveyLang"];
		$surveyAuthorId 		= $_SESSION["sessionUserId"];
		
        // check,if all nedded values have been submitted
        if($surveyName == "" or $surveyAuthor == "" or $surveyDescription == "" or $surveyOutputdir == "")
        {   
            $this->setErrorMessage("no value for surveyName or surveyAuthor or surveyDescription or surveyOutputdir or surveyStartdate or surveyEnddate submitted");
            return false;
        }
        
        // check if this survey already exists
        if(!$this->existsSurvey($surveyName))
        {   
            return false;
        }
                
        // get next free id
        $nextId = $this->db->nextId($cfgTabSurvey);
        if (DB::isError($nextId)) 
        {
            $this->setErrorMessage($result->getMessage());
            return false;
        }
        
        // create insert statement
		$sql = "insert 
			into $cfgTabSurvey 
            (PROJECT_ID_PK, PROJECT_AUTHOR_ID, PROJECT_NAME, PROJECT_AUTHOR, PROJECT_DESCRIPTION, PROJECT_OUTPUTDIR, PROJECT_ACTIVE, PROJECT_STARTDATE, PROJECT_ENDDATE, PROJECT_TIMESTAMP) 
            values 
            ('$nextId', '$surveyAuthorId', '$surveyName', '$surveyAuthor', '$surveyDescription', '$surveyOutputdir', '$surveyActive', '$surveyStartdate', '$surveyEnddate', NOW())";
        //
		$result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
           $this->setErrorMessage($result->getMessage());
           return false;
        }
        
        return $nextId;
    }

    /**
    *	deletes a survey
    *
    *
    *	@access	public
    *	@param	int $surveyId
    *   @return string true/false
    */
    function deleteSurvey($surveyId)
    {
        global $cfgTabSurvey;
        
        // check if all values are submitted
        if($surveyId == "")
        {   
            $this->setErrorMessage("no value for surveyId submitted");
            return false;
        }
        
        // delete survey
        $sql = "delete 
			from $cfgTabSurvey  
            where PROJECT_ID_PK = $surveyId";
		//
        $result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
           $this->setErrorMessage($result->getMessage());
           return false;
        }
        
        return true;
    }
    
    /**
    *	edit a survey
    *
    *	@access	public
    *	@param	array	$surveyArray
    *   @return int $nextId id of the new survey
    */
    function editSurvey($surveyArray)
    {
        global $cfgTabSurvey;
        
        // extract submitted values
        $surveyId               = $surveyArray["surveyId"];
		$surveyName 			= $surveyArray["surveyName"];
		$surveyDescription 		= $surveyArray["surveyDescription"];
		$surveyOutputdir 		= $surveyArray["surveyOutputDir"];
		$surveyStartDate 		= $surveyArray["surveyStartDate"];
		$surveyEndDate 			= $surveyArray["surveyEndDate"];
				
        // check,if all nedded values have been submitted
        if($surveyId == "" or $surveyDescription == "" or $surveyOutputdir == "")
        {   
            $this->setErrorMessage("no value for surveyId or surveyName or surveyDescription or surveyOutputdir ubmitted");
            return false;
        }
        
        // check if this survey exists
        if(!$this->isSurvey($surveyId))
        {   
            return false;
        }
        
        // create insert statement
		$sql = "update 
			$cfgTabSurvey 
			set 
            PROJECT_DESCRIPTION = '$surveyDescription', 
            PROJECT_OUTPUTDIR = '$surveyOutputdir', 
            PROJECT_STARTDATE = '$surveyStartDate', 
            PROJECT_ENDDATE = '$surveyEndDate' 
            where PROJECT_ID_PK = $surveyId";
        //    
        $result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
           $this->setErrorMessage($result->getMessage());
           return false;
        }
        
        return true;
    }
	
	/**
    *	activate or deactivate a survey
    *
    *	@access	public
    *	@param	int	$surveyId
    *	@param	string	$status
    *   @return boolean
    */
	function setSurveyStatus($surveyId, $status)
    {
        global $cfgTabSurvey;

        // create insert statement
		$sql = "update 
			$cfgTabSurvey 
			set 
            PROJECT_ACTIVE = '$status' 
            where PROJECT_ID_PK = $surveyId";

        $result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
           $this->setErrorMessage($result->getMessage());
           return false;
        }
        
        return true;
    }

    /**
    *	deletes permissions for this survey
    *
    *	@access	public
    *	@param	int    $contentTypeId   contenttype id (1 for survey projects)
    *	@param	int    $surveyId        survey project id
    *   @return string true/false
    */
    function deleteSurveyPermissions($contentTypeId, $surveyId)
    {
        global $cfgTabAuthContentToCategory;
        
        $sql = "delete 
			from $cfgTabAuthContentToCategory 
            where 
            CNTNT_TO_CTGRY_CNTNTTYP_ID_FK = '$contentTypeId' 
			and CNTNT_TO_CTGRY_CONTENT_ID = '$surveyId'";
        //
        $result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
           $this->setErrorMessage($result->getMessage());
           return false;
        }
        
        return true;
    }
    
    /**
    *	deletes css styles for this survey
    *
    *	@access	public
    *	@param	int    $surveyId        survey project id
    *   @return string true/false
    */
    function deleteSurveyStyles($surveyId)
    {
        global $cfgSrvyTabCss;
        
        $sql = "delete 
			from $cfgSrvyTabCss 
            where CSSSTYLE_PROJECT_ID = '$surveyId'";
        //
        $result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
           $this->setErrorMessage($result->getMessage());
           return false;
        }
        
        return true;
    }
    
    /**
    *	deletes errors for a survey
    *
    *	@access	public
    *	@param	int    $surveyId        survey project id
    *   @return string true/false
    */
    function deleteSurveyErrorTable($projectId)
    {
		global $cfgSrvyTabError;
		
        // delete survey templates
        $sql = "delete 
			from $cfgSrvyTabError 
			where ERROR_PROJECT_ID = $projectId";
        //
		$result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
           $this->setErrorMessage($result->getMessage());
           return false;
        }
		
		return true;
    }
	
    /**
    *	deletes tan table for a survey
    *
    *	@access	public
    *	@param	int    $surveyId        survey project id
    *   @return string true/false
    */
    function deleteSurveyTemplateTables($projectId)
    {
		global $cfgSrvyTabTmpl2Project;
		
        // delete survey templates
        $sql = "delete 
			from $cfgSrvyTabTmpl2Project 
			where TMPL_TO_PRJCT_PROJECT_ID = $projectId";
        //
		$result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
           $this->setErrorMessage($result->getMessage());
           return false;
        }
		
		return true;
    }
	
	
    /**
    *	deletes tan table for a survey
    *
    *	@access	public
    *	@param	int    $surveyId        survey project id
    *   @return string true/false
    */
    function deleteSurveyTanTables($surveyId)
    {
        $table = "T_SURVEY_SECURITYTAN_".$surveyId;
        $sql = "drop table  $table";
        $result = $this->db->query($sql);
        
        $table = "T_SURVEY_SECURITYTAN_".$surveyId."_seq";
        $sql = "drop table  $table";
        $result = $this->db->query($sql);
                
        $table = "T_SURVEY_TANGROUP_".$surveyId;
        $sql = "drop table  $table";
        $result = $this->db->query($sql);
		
        $table = "T_SURVEY_TANGROUP_".$surveyId."_seq";
        $sql = "drop table  $table";
        $result = $this->db->query($sql);
		
        return true;
    }
    
    /**
    *	deletes result tables for this survey
    *
    *	@access	public
    *	@param	int    $surveyId        survey project id
    *   @return string true/false
    */
    function deleteResultTables($surveyId)
    {
        //delete tables and sequences
        
        $table = "T_SURVEY_FILLED_".$surveyId;
        $sql = "drop table  $table";
        $result = $this->db->query($sql);
               
        $table = "T_SURVEY_FILLED_".$surveyId."_seq";
        $sql = "drop table  $table";
        $result = $this->db->query($sql);
        
        $table = "T_SURVEY_RESULT_".$surveyId;
        $sql = "drop table  $table";
        $result = $this->db->query($sql);
        
        $table = "T_SURVEY_RESULT_".$surveyId."_seq";
        $sql = "drop table  $table";
        $result = $this->db->query($sql);
                
        return true;
    }
    
    /**
    *	empty result tables for this survey
    *
    *	@access	public
    *	@param	int    $surveyId        survey project id
    *   @return string true/false
    */
    function emptyResultTables($surveyId)
    {
        $table = "T_SURVEY_FILLED_".$surveyId;
        $sql = "delete from $table";
        $result = $this->db->query($sql);
              
        $table = "T_SURVEY_RESULT_".$surveyId;
        $sql = "delete from $table";
        $result = $this->db->query($sql);
        
       return true;
    }
    
    /**
    *	get all surveys for overview
    *
    *	@access	public
    *	@param	
    *   @return string true/false
    */
    function getSurveys($projectId, $actualNum = 0, $maxRow, $orderBy, $direction)
    {
        global $cfgTabSurvey;

        // create new permissionFunctions object
        $permission = &new permissionFunctions;
        
		// check if projectId is 0
        if($projectId == "0")
        {   
        	// select all surveys
        	$sql = "select * 
					from $cfgTabSurvey 
				    order by $orderBy $direction 
                    limit $actualNum, $maxRow";

			$result = $this->db->query($sql);
        	if (DB::isError($result)) 
        	{
           	    $this->setErrorMessage($sql."  ".$result->getMessage());
           	    return false;
       	 	}
		}
		else
		{
			// fetch selected survey
			$sql = "select * 
				from $cfgTabSurvey 
				where PROJECT_ID_PK = '$projectId'";
        	//
			$result = $this->db->query($sql);
        	if (DB::isError($result)) 
        	{
               	$this->setErrorMessage($result->getMessage());
               	return false;
       	 	}
			//echo "<br>sql2:$sql<br>";
		}
       	while ($row = $result->fetchRow(DB_FETCHMODE_ASSOC))
        {
            // get Permission
            $permissionMode = $permission->getPermissionMode($_SESSION["sessionUserId"], $row["PROJECT_ID_PK"], 1);
            // permission must be 1 (read)
            if($permissionMode >= 1)
            {
                $getSurveys[] = $row;
            }
        }
        
		return $getSurveys;
    }
	
 	/**
    *	get one survey 
    *
    *	@access	public
    *	@param	
    *   @return string true/false
    */
    function getSurvey($projectId)
    {
        global $cfgTabSurvey;
		
		// create new permissionFunctions object
        $permission = &new permissionFunctions;
		
		// fetch selected survey
		$sql = "select * 
			from $cfgTabSurvey 
			where PROJECT_ID_PK = '$projectId'";
       	//
		$result = $this->db->query($sql);
       	if (DB::isError($result)) 
       	{
              	$this->setErrorMessage($result->getMessage());
              	return false;
      	 }
	
       	while ($row = $result->fetchRow(DB_FETCHMODE_ASSOC))
        {
            // get Permission
            $permissionMode = $permission->getPermissionMode($_SESSION["sessionUserId"], $row["PROJECT_ID_PK"], 1);
            // permission must be 1 (read)
            if($permissionMode >= 1)
            {
                $getSurveys[] = $row;
            }
        }
        
		return $getSurveys;
    }
	
	/**
    *	gets sum of surveys
    *
    *	@access	public
    *	@param	
    *   @return string true/false
    */
	function getSurveysNum($projectId)
	{
		global $cfgSrvyTabSurvey;
        
        // create new permissionFunctions object
        $permission = &new permissionFunctions;
        
        $numRows = "0";
        
		// check if projectId is 0
        if($projectId == 0)
        {   
            // fetch all surveys
        	$sql = "select PROJECT_ID_PK 
				from $cfgSrvyTabSurvey";
			//
        	$result = $this->db->query($sql);
        	if (DB::isError($result)) 
        	{
                return false;
        	}
            while ($row = $result->fetchRow(DB_FETCHMODE_ASSOC))
            {
                // get Permission
                $permissionMode = $permission->getPermissionMode($_SESSION["sessionUserId"], $row["PROJECT_ID_PK"], 1);
                // permission must be 1 (read)
                if($permissionMode >= 1)
                {
                    $numRows++;
                }
            }
        }
		else
		{
            // get Permission
            $permissionMode = $permission->getPermissionMode($_SESSION["sessionUserId"], $projectId, 1);
            // permission must be 1 (read)
            if($permissionMode >= 1)
            {
            	// fetch selected survey
            	$sql = "select 1 
					from $cfgSrvyTabSurvey 
					where PROJECT_ID_PK = '$projectId'";
				// 
            	$result = $this->db->query($sql);
            	if (DB::isError($result)) 
            	{
                    return false;
            	}
                
                $numRows = $result->numRows();
            }
		}
        
        return $numRows;
	}
    
	/**
    *	get all groups in a survey folder or subfolder
    *
    *	@access	public
    *	@param	$projectId  survey project id
    *	@param	$groupId    group id
    *	@param	$parentId   parent id
    *	@param	$actualNum  
    *	@param	$maxRow     
    *	@param	$orderBy    
    *	@param	$direction  
    *   @return array       array with group info
    */
	function getSurveyGroupsForOverview($projectId, $groupId, $parentId, $actualNum, $maxRow, $orderBy, $direction)
    {
		global $cfgSrvyTabForms;
		
        if($parentId == "0")
        {
            // select all groups for this survey
            $sql = "select * 
				from $cfgSrvyTabForms
                where FORM_PROJECT_ID_FK = $projectId 
				and FORM_PARENT_ID = $parentId
                order by $orderBy $direction 
                limit $actualNum, $maxRow";
        }
        else
        {
            // select all groups for this survey
            $sql = "select * 
				from $cfgSrvyTabForms
                where FORM_PROJECT_ID_FK = $projectId 
				and FORM_PARENT_ID = $parentId 
				and ((FORM_ID_PK = $groupId and FORM_NODEGROUP = '0') 
				or (FORM_NODEGROUP = $groupId)) 
                order by $orderBy $direction 
                limit $actualNum, $maxRow";
        }
        $result = $this->db->query($sql);
		if (DB::isError($result)) 
        {
           $this->setErrorMessage($result->getMessage());
           return false;
        }
		while ($row = $result->fetchRow(DB_FETCHMODE_ASSOC))
        {
            $getGroupsForSurvey[] = $row;
			// set order as value to key pk
			$orderNumForId[$row["FORM_ID_PK"]] = $row["FORM_ORDER"];
        }
		// get relative order nums
		$relOrderNumForId = $this->convertToRelativeOrderNum($orderNumForId);
		// insert relorder nums
		for ($i=0;$i<count($getGroupsForSurvey);++$i)
		{
			$getGroupsForSurvey[$i]["REL_ORDER_NUM"] = $relOrderNumForId[$getGroupsForSurvey[$i]["FORM_ID_PK"]];
		}
		return $getGroupsForSurvey;
	}
    
    /**
    *	get all groups in a folder or subfolder with FORM_NODEGROUP = 0
    *
    *	@access	public
    *	@param	$projectId  survey project id
    *	@param	$parentId   parent id
    *   @return array       array with group info
    */
    function getStartSubfolderNodes($projectId, $parentId)
    {
        global $cfgSrvyTabForms;
        
        $sql = "select FORM_ID_PK 
			from $cfgSrvyTabForms
            where FORM_PROJECT_ID_FK = $projectId 
			and FORM_PARENT_ID = $parentId 
			and FORM_NODEGROUP = '0' 
            order by FORM_NAME";
        //
        $result = $this->db->query($sql);
		if (DB::isError($result)) 
        {
           $this->setErrorMessage($result->getMessage());
           return false;
        }
        while ($row = $result->fetchRow(DB_FETCHMODE_ASSOC))
        {
            $rootSubfolderNodes[] = $row;
        }
        
        if($parentId == "0")
        {
            $rootSubfolderNode[] = $rootSubfolderNodes[0];
            return $rootSubfolderNode;
        }
        
        return $rootSubfolderNodes;
    }
	
	/**
    *	get alls groups in a survey
    *
    *	@access	public
    *	@param	$projectId  survey project id
    *   @return array       array with group info
    */
	function getGroupsForSurvey($projectId)
    {
		global $cfgSrvyTabForms;
		
		// select all groups for this survey
		$sql = "select * 
			from $cfgSrvyTabForms
			where FORM_PROJECT_ID_FK = '$projectId'";
		//
		$result = $this->db->query($sql);
		if (DB::isError($result)) 
        {
           $this->setErrorMessage($result->getMessage());
           return false;
        }
		while ($row = $result->fetchRow(DB_FETCHMODE_ASSOC))
        {
            $getGroupsForSurvey[] = $row;
        }
		return $getGroupsForSurvey;
	}
    
    /**
    *	get sum of groups in a survey
    *
    *	@access	public
    *	@param	
    *   @return string true/false
    */
	function getSurveyGroupsNum($projectId, $groupId, $parentId)
	{
		global $cfgSrvyTabForms;
        
        // fetch all users
        if($parentId == "0" or $groupId == "")
        {
            $sql = "select 1 
				from $cfgSrvyTabForms 
                where FORM_PROJECT_ID_FK = $projectId 
				and FORM_PARENT_ID = $parentId"; 
        }
        else
        {
            $sql = "select 1 
				from $cfgSrvyTabForms 
                where FORM_PROJECT_ID_FK = $projectId 
				and FORM_PARENT_ID  = $parentId 
				and ((FORM_ID_PK = $groupId or FORM_NODEGROUP = '0') 
				or (FORM_NODEGROUP = $groupId))";
        }
		//
        $result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
            return false;
        }
        
        $numRows = $result->numRows();
        
        return $numRows;
	}
    
    /**
    *	gets first folder of each subfolder in a group
    *
    *	@access	public
    *	@param	
    *   @return array true/false
    */
	function getStartSubFolders($projectId, $groupId)
    {
        global $cfgSrvyTabForms;
        
        $sql = "select FORM_ID_PK, FORM_NAME 
			from $cfgSrvyTabForms 
            where FORM_PROJECT_ID_FK = '$projectId' 
			and FORM_PARENT_ID  = '$groupId' 
			and FORM_NODEGROUP = '0'";
        //
        $result = $this->db->query($sql);
		if (DB::isError($result)) 
        {
           $this->setErrorMessage($result->getMessage());
           return false;
        }
		while ($row = $result->fetchRow(DB_FETCHMODE_ASSOC))
        {
            $getGroupsForSurvey[] = $row;
        }
        
		return $getGroupsForSurvey;
    } 
    
    /**
    *	get sum of entrys in a table
    *
    *	@access	public
    *	@param	
    *   @return string true/false
    */
	function getTableEntriesNum($tabletoCount,$argumentsToCheck,$checkMethod="AND")
	{
        // fetch all users
        $sql = "select 1 
			from $tabletoCount";
		//check Arguments
		if ($argumentsToCheck!=0)
		{
			foreach($argumentsToCheck as $key => $value)
			{
				if ($tabFctCounter==0)
				{
					$sql .=" where ".$key." = '".$value."'";
				}
				else
				{
					//check for AND/OR statement
					$$checkMethod = strtoupper($checkMethod);
					if ($checkMethod == "OR")
					{
						$sql .=" or ".$key." = '".$value."'";
					}
					else
					{
						$sql .=" and ".$key." = '".$value."'";
					}
				}
			$tabFctCounter=1;
			}
		$tabFctCounter=0;
		}
		$result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
            return 0;
        }
        $numRows = $result->numRows();
        return $numRows;
	}

	/**
    *	gets the survey for one group
    *
    *	@access	public
    *	@param	
    *   @return string true/false
    */
	function getSurveyForGroup($projectId)
	{
	    global $cfgSrvyTabSurvey;
	
		 // select all surveys
        $sql = "select * 
			from $cfgSrvyTabSurvey
			where PROJECT_ID_PK  = '$projectId'";
		//
		$result = $this->db->query($sql);
		//echo "sql:$sql";
        if (DB::isError($result)) 
        {
           $this->setErrorMessage($result->getMessage());
           return false;
        }
		while ($row = $result->fetchRow(DB_FETCHMODE_ASSOC))
        {
            $getSurveyGroup[] = $row;
        }
        
		return $getSurveyGroup;
	}
	
	/**
    *	gets questions for one group
    *
    *	@access	public
    *	@param	
    *   @return string true/false
    */
	function getQuestionsInGroup($groupId)
	{
		global $cfgSrvyTabFormtype;
        
        // fetch all questions
		$sql = "select 1	
			from $cfgSrvyTabFormtype 
            where $cfgSrvyTabFormtype.FORMTYPE_FORM_ID_FK = '$groupId'";
		//
		$result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
            return false;
        }
		
        $numRows = $result->numRows();
        return $numRows;
	}
	
	/**
    *	gets all questions from one group in one surveys for overview
    *
    *	@access	public
    *	@param	
    *   @return string true/false
    */
    function getSurveyQuestionsForGroup($groupId, $actualNum = 0, $maxRow, $orderby, $direction)
    {
		global $cfgSrvyTabFormtype;
		
		// fetch all questions in this group
        $sql = "select * 
           	from $cfgSrvyTabFormtype
           	where 
           	$cfgSrvyTabFormtype.FORMTYPE_FORM_ID_FK = '$groupId'
			order by $orderby $direction 
            limit $actualNum, $maxRow";	
		//
        $result = $this->db->query($sql);
		//echo "sql:$sql";
        if (DB::isError($result)) 
        {
           $this->setErrorMessage($result->getMessage());
           return false;
        }
		while ($row = $result->fetchRow(DB_FETCHMODE_ASSOC))
        {
            $getQuestions[] = $row;
			// set order as value to key pk
			$orderNumForId[$row["FORMTYPE_ID_PK"]] = $row["FORMTYPE_ORDER"];
        }
		// get relative order nums
		$relOrderNumForId = $this->convertToRelativeOrderNum($orderNumForId);
		// insert relorder nums
		for ($i=0;$i<count($getQuestions);++$i)
		{
			$getQuestions[$i]["REL_ORDER_NUM"] = $relOrderNumForId[$getQuestions[$i]["FORMTYPE_ID_PK"]];
		}
		return $getQuestions;
		
    }
	
	/**
    *	convert absolute ordernums to relatives
    *
    *	@access	public
    *	@param	
    *   @return array
    */
	function convertToRelativeOrderNum($arrayToReorder)
	{
		if (!is_array($arrayToReorder))
		{
			return false;
		}
		// sort by absolutes
		asort($arrayToReorder);
		// give relative values
		while (list($key,) = each($arrayToReorder))
		{
			$reorderedArray[$key] = ++$i;
		}
		//return relative order
		return $reorderedArray;
	}
	
	/**
    *	gets surveyId for group in questions overview
    *
    *	@access	public
    *	@param	
    *   @return string true/false
    */
	function getSurveyForGroupInQuestions($groupId,$projectId)
	{
		global $cfgSrvyTabForms;
		global $cfgSrvyTabSurvey;
		
		if ($groupId)
		{
			// fetch all users in this group
	      	$sql = "select $cfgSrvyTabForms.FORM_ID_PK, $cfgSrvyTabForms.FORM_PROJECT_ID_FK, $cfgSrvyTabForms.FORM_NAME,$cfgSrvyTabSurvey.* 
	           	from $cfgSrvyTabForms, $cfgSrvyTabSurvey 
	           	where $cfgSrvyTabForms.FORM_PROJECT_ID_FK = $cfgSrvyTabSurvey.PROJECT_ID_PK 
				and	$cfgSrvyTabForms.FORM_ID_PK = '$groupId'";
		}
		else
		{
			// fetch all users in this group
	      	$sql = "select $cfgSrvyTabForms.FORM_ID_PK, $cfgSrvyTabForms.FORM_PROJECT_ID_FK, $cfgSrvyTabForms.FORM_NAME,$cfgSrvyTabSurvey.* 
	           	from $cfgSrvyTabForms, $cfgSrvyTabSurvey 
	           	where $cfgSrvyTabForms.FORM_PROJECT_ID_FK = $cfgSrvyTabSurvey.PROJECT_ID_PK 
				and	$cfgSrvyTabForms.FORM_PROJECT_ID_FK = '$projectId'";
		}
  		//
		$result = $this->db->query($sql);
        if (DB::isError($result))
        {
           $this->setErrorMessage($result->getMessage());
           return false;
        }
        while ($row = $result->fetchRow(DB_FETCHMODE_ASSOC))
        {
            $getSurveyGroupInQuestions[] = $row;
        }
		return $getSurveyGroupInQuestions;
	}
	
	/**
    *	gets sum of questions for one group
    *
    *	@access	public
    *	@param	
    *   @return string true/false
    */
	function getGroupQuestionsNum($groupId)
    {
       	global $cfgSrvyTabFormtype;
        
        // fetch all questions
		$sql	= "select 1 
            	from $cfgSrvyTabFormtype 
            	where $cfgSrvyTabFormtype.FORMTYPE_FORM_ID_FK  = $groupId";
		//
		$result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
            return false;
        }
        $numRows = $result->numRows();
        return $numRows;
    }
    
    /**
    *	gets all answers to a question
    *
    *	@access	public
    *	@param	string $questionId  question id
    *   @return string true/false
    */
	function getQuestionsAnswers($questionId)
    {
        global $cfgSrvyTabFormelement;
		        
        // fetch all questions
		$sql = "select * 
            from $cfgSrvyTabFormelement
            where FORMELEMENT_FORMTYPE_ID_FK = '$questionId' 
            order by FORMELEMENT_ORDER";
		//
		$result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
            $this->setErrorMessage("getQuestionsAnswers(): ".$result->getMessage()." $sql");
            return false;
        }
        while ($row = $result->fetchRow(DB_FETCHMODE_ASSOC))
        {
            $answerArray[] = $row;
        }
		return $answerArray;
    }
	
	/**
    *	gets detail for question
    *
    *	@access	public
    *	@param	
    *   @return string true/false
    */
	function getQuestionDetail($questionId)
	{
		global $cfgSrvyTabFormtype;	
		
		$sql = "select *
           	from $cfgSrvyTabFormtype 
           	where FORMTYPE_ID_PK  = $questionId";	
		//
        $result = $this->db->query($sql);
		if (DB::isError($result)) 
        {
           $this->setErrorMessage($result->getMessage());
           return false;
        }
		while ($row = $result->fetchRow(DB_FETCHMODE_ASSOC))
        {
            $getQuestionDetail[] = $row;
        }
		return $getQuestionDetail;
	}
	
	/**
    *	adds a new group for a survey
    *
    *	@access	public
    *	@param	
    *   @return string true/false
    */
	function addSurveyGroup($surveyAddGroupArray)
	{
		global $cfgSrvyTabForms;
        
        // extract submitted values
		$projectId	 		= $surveyAddGroupArray["projectId"];
		$groupId	 		= $surveyAddGroupArray["groupId"] != "" ? $surveyAddGroupArray["groupId"] : "0";
		$surveyGroupName 		= $surveyAddGroupArray["surveyGroupName"];
		$surveyGroupText 		= $surveyAddGroupArray["surveyGroupText"];
		$surveySubGroup 		= $surveyAddGroupArray["surveySubGroup"];
		
        // check,if all nedded values have been submitted
        if($surveyGroupName == "" and $projectId == "")
        {   
            $this->setErrorMessage("no value for name or projectId submitted");
            return false;
        }
	   
        // check if this group already exists
        if(!$this->existsSurveyGroup($surveyGroupName, $projectId, $groupId))
        {   
            return false;
        }
        
        // get next order
        if($surveySubGroup == "0")
        {
            $order = "0";
        }
        else
        {
            if($surveySubGroup != "")
            {
                $sql = "select MAX(FORM_ORDER) + 1 
					from $cfgSrvyTabForms 
	                where FORM_PROJECT_ID_FK = '$projectId' 
					and FORM_PARENT_ID = '$groupId' 
					and ((FORM_ID_PK = '$surveySubGroup' and FORM_NODEGROUP = '0') 
					or (FORM_NODEGROUP = '$surveySubGroup'))";
            }
            else
            {
                $sql = "select MAX(FORM_ORDER) + 1 
					from $cfgSrvyTabForms 
	                where FORM_PROJECT_ID_FK = '$projectId' 
					and FORM_PARENT_ID = '$groupId'";
            }
			//
            $order = $this->db->getOne($sql);
            if (DB::isError($order)) 
            {
                $this->setErrorMessage($order->getMessage());
                return false;
            }
        }
        
        // get next free id
        $nextId = $this->db->nextId($cfgSrvyTabForms);
        if (DB::isError($nextId)) 
        {
            $this->setErrorMessage($result->getMessage());
            return false;
        }
        
        // create insert statement
		$sql = "insert 
			into $cfgSrvyTabForms 
            (FORM_ID_PK, FORM_PROJECT_ID_FK, FORM_PARENT_ID, FORM_NODEGROUP, FORM_NAME, FORM_METHOD, FORM_HEADER, FORM_TARGET, FORM_ACTIVE, FORM_ORDER, FORM_INSERTDATE) 
            values 
            ('$nextId', '$projectId', '$groupId', '$surveySubGroup', '$surveyGroupName', 'post', '$surveyGroupText', '', '', '$order', NOW())";
        //
		$result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
           $this->setErrorMessage($result->getMessage());
           return false;
        }
        
        return $nextId;
    }
	
	/**
    *	checks if group in survey exists
    *
    *	@access	public
    *	@param	string	$groupName
    *	@param	string	$parentId
    *   @return string true/false
    */
    function existsSurveyGroup($surveyGroupName, $projectId, $parentId = 0)
    {
        global $cfgSrvyTabForms;
        
        // check if all values are submitted
        if($surveyGroupName == "" or $projectId == "" or $parentId == "")
        {   
            $this->setErrorMessage("no value for groupName, parentId or projectId submitted");
            return false;
        }
        
        // check if group exists
        $sql = "select 1 
			from $cfgSrvyTabForms 
	        where FORM_NAME = '$surveyGroupName' 
			and FORM_PROJECT_ID_FK  = '$projectId' 
			and FORM_PARENT_ID  = '$parentId'";
        //
		$result = $this->db->query($sql);
		//echo "sql:$sql";
        if (DB::isError($result)) 
        {
            $this->setErrorMessage($result->getMessage());
            return false;
        }
        if($result->numRows() > 0)
        {   
            $this->setErrorMessage("the group with this name and ID already exists");
            return false;
        }
        
        return true;
    }
	
	/**
    *	get all data for a group
    *
    *	@access	public
    *	@param	int     $groupId
    *   @return string  true/false
    */
	function getSurveyGroup($groupId)
    {
		global $cfgSrvyTabForms;
		
		// select group
        $sql = "select * 
			from $cfgSrvyTabForms
			where FORM_ID_PK = $groupId";
		//
		$result = $this->db->query($sql);
		if (DB::isError($result)) 
        {
           $this->setErrorMessage($result->getMessage());
           return false;
        }
		while ($row = $result->fetchRow(DB_FETCHMODE_ASSOC))
        {
            $getSurveyGroup[] = $row;
        }
        
		return $getSurveyGroup;
	}
	
	/**
    *	deletes a group in a survey
    *
    *	@access	public
    *	@param	int     $projectId
    *	@param	int     $groupId
    *   @return string  true/false
    */
	function deleteSurveyGroup($projectId, $parentId, $groupId)
    {
		global $cfgSrvyTabForms;
		global $cfgSrvyTabFormelement;
		global $cfgSrvyTabFormtype;
        global $cfgSrvyTabFormEleRules;	
        
		// check if all values are submitted
        if($projectId == "" or $parentId == "" or $groupId == "")
        {  
            $this->setErrorMessage("deleteSurveyGroup(): no value for projectId, parentId or groupId submitted");
            return false;
        }
        
        // check if group is not a root node
        if($parentId != 0)
        {
            // get start node of subfolder
            $sql = "select FORM_NODEGROUP 
				from $cfgSrvyTabForms 
                where 
                FORM_ID_PK = $groupId";
			//
            $nodeGroup = $this->db->getOne($sql); 
            if (DB::isError($nodeGroup)) 
        	{
               	$this->setErrorMessage("deleteSurveyGroup(): ".$nodeGroup->getMessage()." $sql");
               	return false;
       	 	}
            $nodeGroupId = $nodeGroup == "0" ? $groupId : $nodeGroup;
            
            // select all groups in this subfolder
            $sql = "select FORM_ID_PK, FORM_NODEGROUP 
				from $cfgSrvyTabForms 
                where FORM_PROJECT_ID_FK = '$projectId' 
				and FORM_PARENT_ID = '$parentId' 
				and ((FORM_ID_PK = '$nodeGroupId' and FORM_NODEGROUP = '0') 
				or (FORM_NODEGROUP = '$nodeGroupId')) 
                order by FORM_ORDER";
            //
            $result = $this->db->query($sql);
            if (DB::isError($result)) 
        	{
               	$this->setErrorMessage("deleteSurveyGroup(): ".$result->getMessage()." $sql");
               	return false;
       	 	}
           	while ($row = $result->fetchRow(DB_FETCHMODE_ASSOC))
            {
                $formsArray[] = $row;
            }
            // get num of groups in this subfolder
            $formsNum = count($formsArray);
            
            // check the index of the group to delete
            for($i=0;$i<$formsNum;$i++)
            {
                if($formsArray[$i]["FORM_ID_PK"] == $groupId)
                {
                    $groupIndex = $i;
                }
            }
            
            
            // if this group is the first element in a subfolder with more than one element. 
            // we have to change the FORM_NODEGROUP_IDs
            if ($formsArray[$groupIndex]["FORM_NODEGROUP"] == "0" and $formsNum > 1)
            {
                // get the id of second element (which will now be the first element)
                $tempId =  $formsArray[1]["FORM_ID_PK"];
                
                // set the second last element FORM_NODEGROUP to 0
                $sql = "update $cfgSrvyTabForms 
                    set 
					FORM_NODEGROUP = '0' 
                    where FORM_ID_PK = '$tempId'";
                //
                $result = $this->db->query($sql);
                if (DB::isError($result)) 
                {
                   $this->setErrorMessage("deleteSurveyGroup(): ".$result->getMessage()." $sql");
                   return false;
                }
                
                // set all other elements in this subfolder to $tempId
                $sql = "update 
					$cfgSrvyTabForms 
                    set 
					FORM_NODEGROUP = '$tempId' 
                    where FORM_PROJECT_ID_FK = '$projectId' 
					and FORM_PARENT_ID = '$parentId' 
					and FORM_NODEGROUP = '$groupId'";
                //
                $result = $this->db->query($sql);
                if (DB::isError($result)) 
                {
                   $this->setErrorMessage("deleteSurveyGroup(): ".$result->getMessage()." $sql");
                   return false;
                }
            }
            
            // if a switch is set to this group, we have to set him to the new start node of this subfolder
            $sql = "select FORMELEMENT_ID_PK 
				from $cfgSrvyTabFormelement 
                where FORMELEMENT_SWITCHTO = '$groupId'";
			//
            $nodeSwitchFromId = $this->db->getOne($sql); 
            if (DB::isError($nodeSwitchFromId)) 
        	{
               	$this->setErrorMessage($nodeSwitchFromId->getMessage());
               	return false;
       	 	}
            if($nodeSwitchFromId != "")
            {
                // get the id of second element (which will now be the first element)
                $tempId =  $formsArray[1]["FORM_ID_PK"] == "" ? 0 : $formsArray[1]["FORM_ID_PK"];
                                
                // the switch must now point to the new startelement of the subfolder
                $sql = "update 
					$cfgSrvyTabFormelement 
                    set 
					FORMELEMENT_SWITCHTO = '$tempId' 
                    where FORMELEMENT_ID_PK = '$nodeSwitchFromId'";
                //
                $result = $this->db->query($sql);
                if (DB::isError($result)) 
                {
                   $this->setErrorMessage("deleteSurveyGroup(): ".$result->getMessage()." $sql");
                   return false;
                }
            }
        } 
          
        // delete group
        $sql = "delete 
			from $cfgSrvyTabForms 
			where FORM_ID_PK = '$groupId'";
        //
		$result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
           $this->setErrorMessage("deleteSurveyGroup(): ".$result->getMessage()." $sql");
           return false;
        }
		
		// delete formelements
        $sql = "delete 
			from $cfgSrvyTabFormelement 
			where FORMELEMENT_FORM_ID_FK = $groupId";
		//
        $resultEle = $this->db->query($sql);
        if (DB::isError($resultEle)) 
        {
           $this->setErrorMessage("deleteSurveyGroup(): ".$result->getMessage()." $sql");
           return false;
        }
		
		// delete formtypes
        $sql = "delete 
			from $cfgSrvyTabFormtype 
			where FORMTYPE_FORM_ID_FK = $groupId";
		//
        $result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
           $this->setErrorMessage("deleteSurveyGroup(): ".$result->getMessage()." $sql");
           return false;
        }
        
        // delete elementrules
        $sql = "delete 
			from $cfgSrvyTabFormEleRules 
			where ELEMENTRULE_FORM_ID_FK = $groupId";
        //
		$result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
           $this->setErrorMessage("deleteSurveyGroup(): ".$result->getMessage()." $sql");
           return false;
        }
		
        return true;
	}
	
	/**
    *	updates a group in a survey
    *
    *	@access	public
    *	@param	
    *   @return string true/false
    */
	function editSurveyGroup($surveyGroupEditArray)
	{
		global $cfgSrvyTabForms;
        
        // extract submitted values
		$groupId		= $surveyGroupEditArray["groupId"];
		$projectId		= $surveyGroupEditArray["projectId"];
		$surveyGroupName 	= $surveyGroupEditArray["surveyGroupName"];
		$surveyGroupNameOld 	= $surveyGroupEditArray["surveyGroupNameOld"];
		$surveyGroupText 	= $surveyGroupEditArray["surveyGroupText"];
        
        // get parent id
        $parentId = $this->getParent($groupId);
		
        // check,if all nedded values have been submitted
        if($surveyGroupName == "" and $groupId == "")
        {   
            $this->setErrorMessage("no value for name or groupId submitted");
            return false;
        }
		
		// check for groupname (if old group = new group = make update, else check for existing group)
	  	if($surveyGroupName == $surveyGroupNameOld)
		{
	  		// create update statement
			$sql = "update 
				$cfgSrvyTabForms
				set
				FORM_NAME = '$surveyGroupName', 
				FORM_HEADER = '$surveyGroupText'
				where FORM_ID_PK = '$groupId'";
            //
            $result = $this->db->query($sql);
        	if (DB::isError($result)) 
        	{
           	    $this->setErrorMessage($result->getMessage());
           	    return false;
        	}
	  	}
	  	else
		{
        	// check if this group already exists
            if(!$this->existsSurveyGroup($surveyGroupName, $projectId, $parentId))
        	{   
            	return false;
        	}
		
        	// create update statement
			$sql = "update 
				$cfgSrvyTabForms
				set
				FORM_NAME = '$surveyGroupName', 
				FORM_HEADER = '$surveyGroupText'
				where FORM_ID_PK = '$groupId'";
        	//
        	$result = $this->db->query($sql);
        	if (DB::isError($result)) 
        	{
           		$this->setErrorMessage($result->getMessage());
           		return false;
        	}
		}
        return true;
    }
	
	/**
    *	deletes a question in a survey group
    *
    *	@access	public
    *	@param	
    *   @return string true/false
    */
	function deleteSurveyQuestion($questionId)
	{
		global $cfgSrvyTabFormelement;
		global $cfgSrvyTabFormEleRules;
		global $cfgSrvyTabFormtype;	
		
		 // check if all values are submitted
        if($questionId == "")
        {   
            $this->setErrorMessage("no value for questionId submitted");
            return false;
        }
        
        // delete group with all questions
        $sql = "delete 
			from $cfgSrvyTabFormelement 
			where FORMELEMENT_FORMTYPE_ID_FK = '$questionId'";
        //
		$result = $this->db->query($sql);
		if (DB::isError($result)) 
        {
           $this->setErrorMessage($result->getMessage());
           return false;
        }
		
		// delete group with all questions in formelements
        $sql = "delete 
			from $cfgSrvyTabFormtype 
			where FORMTYPE_ID_PK = '$questionId'";
        //
		$resultEle = $this->db->query($sql);
		if (DB::isError($resultEle)) 
        {
           $this->setErrorMessage($result->getMessage());
           return false;
        }
		
		// delete rules
        $sql = "delete 
			from $cfgSrvyTabFormEleRules 
			where ELEMENTRULE_FORMTYPE_ID_FK = '$questionId'";
        //
		$resultEleRules = $this->db->query($sql);
		if (DB::isError($resultEleRules)) 
        {
           $this->setErrorMessage($result->getMessage());
           return false;
        }
        return true;
	}
    
    /**
    *	checks if survey does already exist
    *
    *	@access	public
    *	@param	string	$surveyName
    *   @return string  true/false
    */
    function existsSurvey($surveyName)
    {
        global $cfgTabSurvey;
        
        // check if all values are submitted
        if($surveyName == "")
        {   
            $this->setErrorMessage("existsSurvey: no value for surveyName submitted");
            return false;
        }
        
        // check if survey exists
        $sql = "select 1 
			from $cfgTabSurvey 
	        where PROJECT_NAME = '$surveyName'";
		//
        $result = $this->db->query($sql);
        if($result->numRows() == 1)
        {   
            $this->setErrorMessage("existsSurvey: the survey with this name already exists");
            return false;
        }
        
        return true;
    }
    
    /**
    *	checks if survey does exist
    *
    *	@access	public
    *	@param	string	$surveyId
    *   @return string  true/false
    */
    function isSurvey($surveyId)
    {
        global $cfgTabSurvey;
        
        // check if all values are submitted
        if($surveyId == "")
        {   
            $this->setErrorMessage("isSurvey: no value for surveyId submitted");
            return false;
        }
        
        // check if survey exists
        $sql = "select 1 
			from $cfgTabSurvey 
	        where PROJECT_ID_PK = '$surveyId'";
        //
		$result = $this->db->query($sql);
        if($result->numRows() == 0)
        {   
            $this->setErrorMessage("isSurvey: the survey with this name does not exist");
            return false;
        }
        
        return true;
    }
    
    /**
    *	checks if a table is already created
    *
    *	@access	public
    *	@param	string  $table  name of table
    *   @return string true/false
    */
    function existsTable($table)
    {
        $tableArray = $this->db->getListOf("tables");
        if (DB::isError($test)) 
        {  
           $this->setErrorMessage($result->getMessage());
           return false; 
        }
        
        if(in_array($table, $tableArray))
        {
            return true;
        }
        
        return false;
    }
    
    /**
    *	creates the outpudir for the survey 
    *
    *	@access	public
    *	@param	string $outputDir  name of outputdir
    *   @return string true/false
    */
    function createOutputDir($outputDir)
    {
        global $cfgPathToOutput;
        
        $path =  $cfgPathToOutput."/".$outputDir;  
        if (!file_exists($path)) 
        { 
            mkdir($path, 0777); 
        } 
        $dir = dir($path);
    }
    
    /**
    *	copies the survey start files into output folder
    *
    *	@access	public
    *	@param	string  $outputDir  name of outputDir
    *   @return string true/false
    */
    function copySurveyAddFiles($outputDir)
    {
        global $cfgPathToOutput;
        
		if(!copy($cfgPathToOutput."/default.groupdetail.php", $cfgPathToOutput."/".$outputDir."/groupdetail.php"))
        {   
            $this->setErrorMessage("copySurveyAddFiles(): can't create groupdetail.php");
            return false;
        }
		
        if(!copy($cfgPathToOutput."/default.preview.php", $cfgPathToOutput."/".$outputDir."/preview.php"))
        {   
            $this->setErrorMessage("copySurveyAddFiles(): can't create preview.php");
            return false;
        }
		
        if(!copy($cfgPathToOutput."/basic.css", $cfgPathToOutput."/".$outputDir."/basic.css"))
        {   
            $this->setErrorMessage("copySurveyAddFiles(): can't create basic.css");
            return false;
        }

        if(!copy($cfgPathToOutput."/default.deactivated.php", $cfgPathToOutput."/".$outputDir."/deactivated.php"))
        {   
            $this->setErrorMessage("copySurveyCreateFiles(): can't create deactivated.php");
            return false;
        }
        
        return true;
    }
	
    /**
    *	copies the survey create files into output folder
    *
    *	@access	public
    *	@param	int     $projectId  project id
    *	@param	string  $outputDir  name of outputDir
    *   @return string true/false
    */
    function copySurveyCreateFiles($projectId, $outputDir, $templateArray, $lang="deutsch")
    {
        global $cfgPathToOutput;
        
        // copy default index template
        if(!copy($cfgPathToOutput."/default.index.php", $cfgPathToOutput."/".$outputDir."/index.php"))
        {   
            $this->setErrorMessage("copySurveyCreateFiles(): can't create index.php");
            return false;
        }
        
        // check if a login template exists
        if($templateArray[0]["TMPL_TO_PRJCT_LOGIN_ID"] == "0" or $templateArray[0]["TMPL_TO_PRJCT_LOGIN_ID"] == "")
        {
            // copy default login template
            if(!copy($cfgPathToOutput."/default.login.php", $cfgPathToOutput."/".$outputDir."/login.php"))
            {   
                $this->setErrorMessage("copySurveyAddFiles(): can't create login.php");
                return false;
            }
        }
        
        // check if a form template exists
        if($templateArray[0]["TMPL_TO_PRJCT_FORM_ID"] == "0" or $templateArray[0]["TMPL_TO_PRJCT_FORM_ID"] == "")
        {
            // copy default form template
            if(!copy($cfgPathToOutput."/default.form.php", $cfgPathToOutput."/".$outputDir."/form.php"))
            {   
                $this->setErrorMessage("copySurveyAddFiles(): can't create form.php");
                return false;
            }
            // copy default preview template
            if(!copy($cfgPathToOutput."/default.preview.php", $cfgPathToOutput."/".$outputDir."/preview.php"))
            {   
                $this->setErrorMessage("copySurveyAddFiles(): can't create preview.php");
                return false;
            }
        }
        
        // check if a confirm template exists
        if($templateArray[0]["TMPL_TO_PRJCT_CONFIRM_ID"] == "0" or $templateArray[0]["TMPL_TO_PRJCT_CONFIRM_ID"] == "")
        {
            // copy default confirm template
            if(!copy($cfgPathToOutput."/default.confirm.php", $cfgPathToOutput."/".$outputDir."/confirm.php"))
            {   
                $this->setErrorMessage("copySurveyAddFiles(): can't create form.php");
                return false;
            }
        }
		
        $xmlString = "<?xml version='1.0' ?><project><id>$projectId</id><lang>$lang</lang></project>";
        $fp = fopen ($cfgPathToOutput."/".$outputDir."/survey.xml", "w");
        if(!$fp)
        {   
            $this->setErrorMessage("copySurveyCreateFiles(): can't create survey.xml");
            return false;
        }
        if(!fwrite($fp, $xmlString))
        {   
            $this->setErrorMessage("copySurveyCreateFiles(): can't write into survey.xml");
            return false;
        }
        fclose($fp);
        
        return true;
    }
    
    /**
    *	checks if a template is set to the project
    *
    *	@access	public
    *	@param	string     $fileContent
    *   @return string/false
    */
	function isTemplateSet($projectId)
	{
        global $cfgSrvyTabTmpl2Project;
        
		$sql = "select * 
			from $cfgSrvyTabTmpl2Project 
            where TMPL_TO_PRJCT_PROJECT_ID = '$projectId'";
        //
		$result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
           $this->setErrorMessage($result->getMessage());
           return false;
        }
        $templateArray = array();
		while ($row = $result->fetchRow(DB_FETCHMODE_ASSOC))
        {
            $templateArray[] = $row;
        }
		return $templateArray;
	}
    
    /**
    *	deletes output folder with all content
    *
    *	@access	public
    *	@param	string  $outputDir  name of outputDir
    *   @return string true/false
    */
    function deleteSurveyFolder($outputdir)
    {
        global $cfgPathToOutput;
        
		if($outputdir)
		{
        	$path =  $cfgPathToOutput."/".$outputdir;
            if (file_exists($path)) 
            {
                // delete folder
                //exec("rm -rf $path");				
				$this->deltree ($path); // base directory is now empty
				rmdir ($path); // base directory is now gone.

            }
		}
                      
        return true;
    }
	
	/**
    *	deletes folders with all content
    *
    *	@access	public
    *	@param	string  $dir
    */
	function deltree($dir){
		$d = dir($dir);
		while($f = $d->read() )
		{
			if($f != "." && $f != "..")
			{
				if(is_dir($dir.$f))
				{
					$this->deltree($dir.$f."/");
					rmdir($dir.$f);
				} // if
				if(is_file($dir.$f))
				unlink($dir.$f);
			}// if
		}// while
		$d->close();
	}
    
    /**
    *	changes the order of question groups
    *
    *	@access	public
    *	@param	int  $surveyId  survey id
    *	@param	int  $parentId  parent id
    *	@param	int  $groupId   group id
    *	@param	int  $order     new position
    *   @return string true/false
    */
    function moveGroupToOrder($surveyId, $parentId, $groupId, $order)
    {
    	global $cfgSrvyTabForms;
        global $cfgSrvyTabFormelement;
		
		 // check if all values are submitted
        if($surveyId == "" || $parentId == "" || $groupId == "" || $order == "")
        {   
            $this->setErrorMessage("moveGroupToOrder(): no value for surveyId, parentId, groupId or order submitted");
            return false;
        }
        
        // check if group is not a root node
        if($parentId != 0)
        {
            // get start node of subfolder
            $sql = "select FORM_NODEGROUP 
				from $cfgSrvyTabForms 
                where FORM_ID_PK = '$groupId'";
            //
            $nodeGroup = $this->db->getOne($sql); 
            if (DB::isError($nodeGroup)) 
        	{
               	$this->setErrorMessage("moveGroupToOrder(): ".$nodeGroup->getMessage()." $sql");
               	return false;
       	 	}
            $nodeGroupId = $nodeGroup == "0" ? $groupId : $nodeGroup;
            
            // select all groups in this subfolder
            $sql = "select FORM_ID_PK, FORM_ORDER 
				from $cfgSrvyTabForms 
                where FORM_PROJECT_ID_FK = '$surveyId' 
				and FORM_PARENT_ID = '$parentId' 
				and ((FORM_ID_PK = '$nodeGroupId' or FORM_NODEGROUP = '0') 
				or (FORM_NODEGROUP = '$nodeGroupId')) 
                order by FORM_ORDER";
            //
            $result = $this->db->query($sql);
        	if (DB::isError($result)) 
        	{
               	$this->setErrorMessage("moveGroupToOrder(): ".$result->getMessage()." $sql");
               	return false;
       	 	}
           	while ($row = $result->fetchRow(DB_FETCHMODE_ASSOC))
            {
                $formsArray[] = $row;
            }
            //print_r($formsArray);
            
            // get num of groups in this subfolder
            $formsNum = count($formsArray);
            
            // if there less than 2 elements we don't have t reorder groups 
            if($formsNum < 2)
            {
                return true;
            }
            
            // if the first element is moved to first position, we can stop here
            if($groupId == $nodeGroupId and $order == $formsArray[0]["FORM_ORDER"])
            {
                return true;
            }
            
            // check if the moved element is the first node
            if($groupId == $nodeGroupId)
            {
                //echo "is first<br>\n";
                // get the second element in the subfolder
                $newStartFolderId = $formsArray[1]["FORM_ID_PK"];
                
                // insert FORM_NODEGROUP = 0 for new start element
                $sql = "update 
					$cfgSrvyTabForms 
                    set 
					FORM_NODEGROUP = '0' 
                    where 
                    FORM_ID_PK = '$newStartFolderId'";
                //
                $result = $this->db->query($sql);
                if (DB::isError($result)) 
                {
                   $this->setErrorMessage("moveGroupToOrder(): ".$result->getMessage()." $sql");
                   return false;
                }
                
                // insert id of new startfolder in FORM_NODEGROUP for all other folders in this subgroup
                $sql = "update 
					$cfgSrvyTabForms 
                    set 
					FORM_NODEGROUP = '$newStartFolderId' 
                    where FORM_ID_PK != '$newStartFolderId' 
					and FORM_PROJECT_ID_FK = '$surveyId' 
					and FORM_PARENT_ID = '$parentId' 
					and (FORM_ID_PK = '$nodeGroupId' or FORM_NODEGROUP = '$nodeGroupId')";
                //
                $result = $this->db->query($sql);
                if (DB::isError($result)) 
                {
                   $this->setErrorMessage("moveGroupToOrder(): ".$result->getMessage()." $sql");
                   return false;
                }
                
                // new nodeGroupId
                $nodeGroupId = $newStartFolderId;
                
                // check if there was a switch set to this element
                // if a switch was set to this group, we have to set it to the new start node of this subfolder
                $sql = "select FORMELEMENT_ID_PK 
					from $cfgSrvyTabFormelement 
                    where FORMELEMENT_SWITCHTO = '$groupId'";
                //
				$nodeSwitchFromId = $this->db->getOne($sql); 
                if (DB::isError($nodeSwitchFromId)) 
            	{
                   	$this->setErrorMessage("moveGroupToOrder(): ".$nodeSwitchFromId->getMessage()." $sql");
                   	return false;
           	 	}
                
                if($nodeSwitchFromId != "")
                {
                    //echo "was switch<br>\n";
                    // the switch must now point to the new startelement of the subfolder
                    $sql = "update 
						$cfgSrvyTabFormelement 
                        set 
						FORMELEMENT_SWITCHTO = '$newStartFolderId' 
                        where FORMELEMENT_ID_PK = '$nodeSwitchFromId'";
                    //
                    $result = $this->db->query($sql);
                    if (DB::isError($result)) 
                    {
                       $this->setErrorMessage("moveGroupToOrder(): ".$result->getMessage()." $sql");
                       return false;
                    }
                }
            }
            // check if the new position of the element is the start element
            if($order == $formsArray[0]["FORM_ORDER"])
            {
               //echo "wird start<br>\n";
               // insert FORM_NODEGROUP = 0 for new start element
                $sql = "update 
					$cfgSrvyTabForms 
                    set 
					FORM_NODEGROUP = '0' 
                    where FORM_ID_PK = '$groupId'";
                //
                $result = $this->db->query($sql);
                if (DB::isError($result)) 
                {
                   $this->setErrorMessage("moveGroupToOrder(): ".$result->getMessage()." $sql");
                   return false;
                }
                
                // insert id of new startfolder in FORM_NODEGROUP for all other folders in this subgroup
                $sql = "update 
					$cfgSrvyTabForms 
                    set 
					FORM_NODEGROUP = '$groupId' 
                    where FORM_ID_PK != '$groupId' 
					and FORM_PROJECT_ID_FK = '$surveyId' 
					and FORM_PARENT_ID = '$parentId' 
					and ((FORM_ID_PK = '$nodeGroupId' 
					and FORM_NODEGROUP = '0') 
					or (FORM_NODEGROUP = '$nodeGroupId'))";
                //
                $result = $this->db->query($sql);
                if (DB::isError($result)) 
                {
                   $this->setErrorMessage("moveGroupToOrder(): ".$result->getMessage()." $sql");
                   return false;
                }
                
                // check if any switch points to the former first element
                $sql = "select FORMELEMENT_ID_PK 
					from $cfgSrvyTabFormelement 
                    where FORMELEMENT_SWITCHTO = '$nodeGroupId'";
                //
				$nodeSwitchFromId = $this->db->getOne($sql); 
                if (DB::isError($nodeSwitchFromId)) 
            	{
                   	$this->setErrorMessage("moveGroupToOrder(): ".$nodeSwitchFromId->getMessage()." $sql");
                   	return false;
           	 	}
                if($nodeSwitchFromId != "")
                {
                    //echo "is switch<br>\n";
                    // the switch must now point to the new startelement of the subfolder
                    $sql = "update 
						$cfgSrvyTabFormelement 
                        set 
						FORMELEMENT_SWITCHTO = '$groupId' 
                        where FORMELEMENT_ID_PK = '$nodeSwitchFromId'";
                    //
                    $result = $this->db->query($sql);
                    if (DB::isError($result)) 
                    {
                       $this->setErrorMessage("moveGroupToOrder(): ".$result->getMessage()." $sql");
                       return false;
                    }
                }
                // $groupId is now nes start element ID
                $nodeGroupId = $groupId;
            }
        }
        
        if($parentId != 0)
        {
            $sql = "update 
				$cfgSrvyTabForms 
                set FORM_ORDER = (FORM_ORDER + 1) 
                where FORM_ORDER >= $order 
				and FORM_PROJECT_ID_FK = $surveyId 
				and FORM_PARENT_ID = $parentId 
				and((FORM_ID_PK = $nodeGroupId or FORM_NODEGROUP = '0') 
				or (FORM_NODEGROUP = $nodeGroupId))";
        }
        else
        {
            $sql = "update 
				$cfgSrvyTabForms 
                set 
                FORM_ORDER = (FORM_ORDER + 1) 
                where FORM_ORDER >= $order 
				and FORM_PROJECT_ID_FK = $surveyId 
				and FORM_PARENT_ID = '0'";
        }
        //
        $result = $this->db->query($sql);
        if (DB::isError($result)) 
        {   
            $this->setErrorMessage("moveGroupToOrder(): ".$result->getMessage()." $sql");
            return false;
        }
        
        $sql = "update 
			$cfgSrvyTabForms 
            set 
            FORM_ORDER = '$order' 
            where FORM_ID_PK = '$groupId'";
        //
        $result = $this->db->query($sql);
        if (DB::isError($result)) 
        {   
            $this->setErrorMessage("moveGroupToOrder(): ".$result->getMessage()." $sql");;
            return false;
        }

		return true;
    }

    /**
    *	moves question groups to the end
    *
    *	@access	public
    *	@param	int  $surveyId  survey id
    *	@param	int  $parentId  parent id
    *	@param	int  $groupId   group id
    *   @return string true/false
    */
    function moveGroupToEnd($surveyId, $parentId, $groupId)
    {
    	global $cfgSrvyTabForms;
        global $cfgSrvyTabFormelement;

		// check if all values are submitted
        if($surveyId == "" || $parentId == ""|| $groupId == "")
        {   
            $this->setErrorMessage("moveGroupToEnd(): no value for surveyId, parentId or groupId submitted");
            return false;
        }
        
        // check if group is a root node
        if($parentId == 0)
        {
            // select all groups in this subfolder
            $sql = "select FORM_ORDER 
				from $cfgSrvyTabForms 
                where FORM_PROJECT_ID_FK = '$surveyId' 
				and FORM_PARENT_ID = '$parentId' 
                order by FORM_ORDER";
			//
            $result = $this->db->query($sql);
        	if (DB::isError($result)) 
        	{
               	$this->setErrorMessage("moveGroupToEnd(): ".$result->getMessage()." $sql");
               	return false;
       	 	}
           	while ($row = $result->fetchRow(DB_FETCHMODE_ASSOC))
            {
                $formsArray[] = $row;
            }
            
            // get num of groups in this subfolder
            $formsNum = count($formsArray);
            
            // if there less than 2 elements we don't have t reorder groups 
            if($formsNum < 2)
            {
                return true;
            }
            
            // get FORM_ORDER from second last element and add 1. Thhis is the new FORM_ORDER of the element moved to the end
            $newFormsOrder = $formsArray[$formsNum-1]["FORM_ORDER"] + 1;
            
        }
        else
        { 
           
	    // get start node of subfolder
            $sql = "select FORM_NODEGROUP 
				from $cfgSrvyTabForms 
                where FORM_ID_PK = $groupId";
            //
			$nodeGroup = $this->db->getOne($sql); 
            if (DB::isError($nodeGroup)) 
        	{
               	$this->setErrorMessage("deleteSurveyGroup(): ".$nodeGroup->getMessage()." $sql");
               	return false;
       	 	}
            $nodeGroupId = $nodeGroup == "0" ? $groupId : $nodeGroup;
              
            // select all groups in this subfolder
            $sql = "select FORM_ID_PK, FORM_NODEGROUP, FORM_ORDER 
				from $cfgSrvyTabForms 
                where FORM_PROJECT_ID_FK = $surveyId 
				and FORM_PARENT_ID = $parentId 
				and ((FORM_ID_PK = $nodeGroupId or FORM_NODEGROUP = 0) 
				or (FORM_NODEGROUP = $nodeGroupId)) 
                order by FORM_ORDER";
            //
	    
	    
            $result = $this->db->query($sql);
        	if (DB::isError($result)) 
        	{
               	$this->setErrorMessage("moveGroupToEnd(): ".$result->getMessage()." $sql");
               	return false;
       	 	}
           	while ($row = $result->fetchRow(DB_FETCHMODE_ASSOC))
            {
                $formsArray[] = $row;
            }
            //print_r($formsArray);
            
            // get num of groups in this subfolder
            $formsNum = count($formsArray);
            
            // if there less than 2 elements we don't have t reorder groups 
            if($formsNum < 2)
            {
                return true;
            }
            
            // get new FORM_ORDER for the group which is going to be moved to the end
            $newFormsOrder = $formsArray[$formsNum-1]["FORM_ORDER"] + 1;
                    
            // if the group, which is moved to the end is the start element in this subfolder (FORM_NODEGROUP = 0),
            // we have to change all FORM_NODEGROUP entries in this subfolder.
            if($formsArray[0]["FORM_NODEGROUP"] == 0)
            {
		//echo "is start element<br>\n";
                // check which folder is now the start element of this subgroup
                // and get its id
                $newStartFolderId = $formsArray[1]["FORM_ID_PK"];
                
                // insert FORM_NODEGROUP = 0 for new start element
                $sql = "update 
					$cfgSrvyTabForms 
                    set 
					FORM_NODEGROUP = '0' 
                    where FORM_ID_PK = '$newStartFolderId'";
                //
                $result = $this->db->query($sql);
                if (DB::isError($result)) 
                {
                   $this->setErrorMessage("moveGroupToEnd(): ".$result->getMessage()." $sql");
                   return false;
                }
                
                // insert id of new startfolder in FORM_NODEGROUP for all other folders in this subgroup
                $sql = "update 
					$cfgSrvyTabForms 
                    set 
					FORM_NODEGROUP = '$newStartFolderId' 
                    where FORM_ID_PK != '$newStartFolderId' 
					and FORM_PROJECT_ID_FK = '$surveyId' 
					and FORM_PARENT_ID = '$parentId' 
					and (FORM_ID_PK = '$nodeGroupId' or FORM_NODEGROUP = '$nodeGroupId')";
                //
                $result = $this->db->query($sql);
                if (DB::isError($result)) 
                {
                   $this->setErrorMessage("moveGroupToEnd(): ".$result->getMessage()." $sql");
                   return false;
                }
            }
            
            // if a switch was set to this group, we have to set it to the new start node of this subfolder
            $sql = "select FORMELEMENT_ID_PK 
				from $cfgSrvyTabFormelement 
                where FORMELEMENT_SWITCHTO = '$groupId'";
            //
            $nodeSwitchFromId = $this->db->getOne($sql); 
            if (DB::isError($nodeSwitchFromId)) 
        	{
               	$this->setErrorMessage("moveGroupToEnd(): ".$nodeSwitchFromId->getMessage()." $sql");
               	return false;
       	 	}
            if($nodeSwitchFromId != "")
            {
                // get the id of second element (which will now be the first element)
                $tempId =  $formsArray[1]["FORM_ID_PK"];
                
                // the switch must now point to the new startelement of the subfolder
                $sql = "update 
					$cfgSrvyTabFormelement 
                    set 
					FORMELEMENT_SWITCHTO = '$tempId' 
                    where FORMELEMENT_ID_PK = '$nodeSwitchFromId'";
                //
                $result = $this->db->query($sql);
                if (DB::isError($result)) 
                {
                   $this->setErrorMessage("moveGroupToEnd(): ".$result->getMessage()." $sql");
                   return false;
                }
            }
        }
        
        // udate FORM_ORDER
        $sql = "update 
			$cfgSrvyTabForms 
            set 
            FORM_ORDER = '$newFormsOrder' 
            where FORM_ID_PK = '$groupId'";
        //
		$result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
            $this->setErrorMessage("moveGroupToEnd(): ".$result->getMessage()." $sql");
            return false;
        }

		return true;
    }
    
    /**
    *	checks if node has childs
    *
    *	@access	public
     *	@param	int     $id   
    *   @return string  true/false
    */
    function isNode($id)
    {
        global $cfgSrvyTabForms;
        
        $sql = "select 1 
			from $cfgSrvyTabForms 
			where FORM_PARENT_ID = '$id'";
        //
        $result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
            return "Fehler. ".$this->db->getMessage();
        }
        
        if ($result->numRows()  > 0)
        {
            return true;
        }
                
        return  false;
    }
    
    /**
    *	checks if node has childs and returns array with childs
    *
    *	@access	public
     *	@param	int     $id   group id
    *   @return array   array with child id and name
    */
    function getChilds($groupId)
    {
        global $cfgSrvyTabForms;
        
        $sql = "select FORM_ID_PK 
			from $cfgSrvyTabForms 
			where FORM_PARENT_ID = '$groupId'";
        //
		$result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
              return "Fehler. ".$this->db->getMessage();
        }
        
        while ($row = $result->fetchRow(DB_FETCHMODE_ASSOC))
        {
            $childArray[] = $row;
        }
                
        return  $childArray;
    }
    
    /**
    *	get parent id
    *
    *	@access	public
     *	@param	int     $groupId   group id
    *   @return array   array with child id and name
    */
    function getParent($groupId)
    {
        global $cfgSrvyTabForms;
        
        $sql = "select FORM_PARENT_ID  
            from $cfgSrvyTabForms 
            where FORM_ID_PK = '$groupId'";
		//
        $parentId = $this->db->getOne($sql);
        if (DB::isError($parentId)) 
        {
           $this->setErrorMessage($parentId->getMessage());
           return false;
        }
        
        return $parentId;
    }


    /**
    *	check for concurring nodegroups
    *
    *	@access	public
    *	@param	int     $j  index
    *   @param  array   $surveyGroupOverviewArray Groupdata to check
    *   @return array   array with child id and name
    */
	function checkForConcurringNodes($surveyGroupOverviewArray,$j)
	{
		global $cfgSrvyTabForms;
		
		//print_r($surveyGroupOverviewArray);
		//echo "<br><br<b>>$j</b>";
		$parentId = $surveyGroupOverviewArray[0]["FORM_PARENT_ID"];
        // select all parent nodegroups
        $sql = "select FORM_ID_PK 
				from $cfgSrvyTabForms 
                where FORM_PARENT_ID = $parentId
				and FORM_NODEGROUP = 0";
        //
        $result = $this->db->query($sql);
    	if (DB::isError($result)) 
    	{
           	$this->setErrorMessage("checkForConcurringNodes(): ".$result->getMessage()." $sql");
           	return false;
   	 	}
       	while ($row = $result->fetchRow(DB_FETCHMODE_ASSOC))
        {
            $parentsArray[] = $row["FORM_ID_PK"];
        }
		// select all elements for parent nodegroups
		while ( list(,$parentGroupId) = each($parentsArray) )
		{
	        $sql = "select FORM_ID_PK 
					from $cfgSrvyTabForms 
	                where FORM_NODEGROUP = $parentGroupId 
					order by FORM_ORDER";
	        //
	        $result = $this->db->query($sql);
	    	if (DB::isError($result)) 
	    	{
	           	$this->setErrorMessage("checkForConcurringNodes(): ".$result->getMessage()." $sql");
	           	return false;
	   	 	}
			// build array of ids in real order
			$nodesArray[] = $parentGroupId;
	       	while ($row = $result->fetchRow(DB_FETCHMODE_ASSOC))
	        {
	            $nodesArray[] = $row["FORM_ID_PK"];
	        }
		}
		// check which form chosen and return index
		$idToProcess = $surveyGroupOverviewArray[$j]["FORM_ID_PK"];
		for ($i=0;$i<count($nodesArray);$i++)
		{
			if($nodesArray[$i] == $idToProcess)
			{	
				break;
			}
		}
		//print_r($nodesArray);
		//echo"<br><br><b>".$i."</b>";
		return $i;
	}
	
    /**
    *	check for default rule messages
    *
    *	@access	public
    *	@param	int     projectId  index
    *   @return array   default messages
    */
	function checkForDefaultRuleMessages($projectId)
	{
		global  $cfgSrvyTabError;
		
		// check for default messages
		//
		// get defaultRulesMessages if set for project
        $sql = "select ERROR_LANG, ERROR_TYPE, ERROR_MESSAGE 
			from $cfgSrvyTabError 
			where ERROR_PROJECT_ID = '$projectId'";
		//
		$result = $this->db->query($sql);
		if (DB::isError($result)) 
		{
			$this->errorMessage = $result->getMessage()."SQL: $sql";
			return false;
		}
		if($result->numRows() > 0)
		{
			while ($row = $result->fetchRow(DB_FETCHMODE_ASSOC))
	        {
				$existingRules[$row["ERROR_LANG"]][$row["ERROR_TYPE"]] = $row["ERROR_MESSAGE"];
			}
		}
		// else get global defaultRulesMessages
		else
		{
			// sql
	        $sql = "select ERROR_LANG, ERROR_TYPE, ERROR_MESSAGE 
				from $cfgSrvyTabError 
				where ERROR_PROJECT_ID = '0'";
			//
			$result = $this->db->query($sql);
			if (DB::isError($result)) 
			{
				$this->errorMessage = $result->getMessage()."SQL: $sql";
				return false;
			}
			while ($row = $result->fetchRow(DB_FETCHMODE_ASSOC))
	        {
				$existingRules[$row["ERROR_LANG"]][$row["ERROR_TYPE"]] = $row["ERROR_MESSAGE"];
			}
		}
		
		// return default messages
		return $existingRules;
	}
	
} // end of class surveyFunctions
?>
