<?php
// +----------------------------------------------------------------------+
// | sa template module - template upload management                      |
// +----------------------------------------------------------------------+
// | Copyright (c) 2002 serie a - Digitale Medien und Systeme GmbH        |
// +----------------------------------------------------------------------+
// | Authors: Jrn Allert <allert@serie-a.de>                             |
// |          Rene Renk <renk@serie-a.de>                                 |
// +----------------------------------------------------------------------+
// | Description: Basic functions for upload management                   |
// +----------------------------------------------------------------------+
//
// $Id: tmpl.assessment.inc.php,v 1.1.1.1 2004/07/15 13:34:53 opensurveypilot Exp $

// require all configuration files and libraries
require_once($cfgPathToProjectAdmin."/config/config.inc.php"); // common config file // common config file
require_once($cfgPathToConf."/file.inc.php"); // project files
require_once($cfgPathToTmplFiles."/lib/upload/tmpl.upload.inc.php"); // class feeding pear upload
require_once ("DB.php"); // PEAR::DB
require_once ("Validate.php"); // PEAR::Validate

/**
* tmplFunctions class
*
* Template Management Class
*
* @author  Jrn Allert <allert@serie-a.de>
* @author  Rene Renk <renk@serie-a.de>
* @access  public
*/
class tmplFunctions
{
    /**
    *	handle for db connection
    *	@var    string  $db
    */
    var $db = "";
    
    /**
    *	error message
    *	@var    string  $errorMessage
    */
    var $errorMessage;
    
    /**
    *	create new tmplFunctions object 
    *
    *	@access	public
    */
    function tmplFunctions()
    {
		// global variables
		global $cfgTabAssessmentCategory;
		global $cfgTabAssessmentContent;
		
		global $cfgPathToAssessmentTmpl;
		
		// set variables
		$this->tmplTabCategory	=	$cfgTabAssessmentCategory;
		$this->tmplTabContent	=	$cfgTabAssessmentContent;
		
		$this->pathToOutputTmpl	=	$cfgPathToAssessmentTmpl;
		
		// free errormessage
        $this->errorMessage = "";
        // connect to db
        $this->db = DB::connect(DSN, true);
        if (DB::isError($db)) 
        {    
            $this->setErrorMessage($db->getMessage());
            return false;
        }
    }
    
    /**
    *	sets error message
    *
    *	@access	private
    *	@param	string	$err
    */
    function setErrorMessage($err)
    {
        $this->errorMessage = $err;
    }
    
    /**
    *	adds a category
    *
    *	@access	public
    *	@param	array	$categoryArray
    *   @return int $nextId id of the new category
    */
    function addCategory($categoryArray)
    {
        // extract submitted values
        $name 		= $categoryArray["name"];
        $parentId 	= $categoryArray["parentId"];
        // check,if all nedded values have been submitted
        if($name == "" or $parentId == "")
        {   
            $this->setErrorMessage("no value for name or parentId submitted");
            return false;
        }
        
        // check if this category already exists
        if(!$this->existsCategory($name, $parentId))
        {
            return false;
        }
        
        // check if parentId exists
        if($parentId != "0" and $parentId != "")
        {
            if(!$this->isCategory($parentId))
            {
                $this->setErrorMessage("There is no parent category with this id: $parentId.");
                return false;
            }
        }
        
        // get next free id
        $nextId = $this->db->nextId($this->tmplTabCategory);
        if (DB::isError($nextId)) 
        {
            $this->setErrorMessage($result->getMessage());
            return false;
        }
        
	    // create insert statement
        $sql = "insert 
			into $this->tmplTabCategory 
            (CATEGORY_ID_PK, CATEGORY_NAME, CATEGORY_PARENT_ID, CATEGORY_TIME) 
            values 
            ($nextId, '$name', '$parentId', NOW())";
		//									 
        $result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
           $this->setErrorMessage($result->getMessage());
           return false;
        }
        
        return $nextId;
    }	

    /**
    *	deletes a category
    *
    *   use emptyCategory($catId), if you want to delete all content in this category // 021208-rr not yet implemented
    *
    *	@access	public
    *	@param	int $catId
    *   @return string true/false
    */
    function deleteCategory($categoryId)
    {
	global $lang_script;
        // check if all values are submitted
        if($categoryId == "")
        {   
            $this->setErrorMessage("no value for catId submitted");
            return false;
        }
        // check if categiory exists
        if(!$this->isCategory($categoryId))
        {
            return false;
        } 
	    // get path
		$parentInfo = array();
		$parentInfo = $this->getParentInfoRecursive($categoryId, &$data);
		//	
		if (!$parentInfo)
		{
			$this->setErrorMessage("No path in Function:  deleteCategory");
			return false;
		}
        //build path
        $reverseParentInfo = array_reverse($parentInfo);
        for($j=0;$j<count($reverseParentInfo);$j++)
        {
            $categoryPath .= "/".$reverseParentInfo[$j]["CATEGORY_NAME"];
        }
		// make secure path given to protect root
		if(!$categoryPath)
		{
			$this->setErrorMessage("No path in Function:  deleteCategory");
			return false;
		}
		// add complete path to content Array
		$toDeletePath = $this->pathToOutputTmpl.$categoryPath;
		//////////////////////////////////////////////
		//
		// local recursive function to delete path
		//
		function deleteAllFromPath($dir)
		{
			global $lang_script;
			// delete everything in the directory
			if($handle = @opendir($dir)) 
			{
				while(($file = readdir($handle)) != false) 
				{
					if($file == "." || $file == "..")
					{
						continue;
					}
					if(is_dir($dir."/".$file))
					{
						// call self for this directory
						deleteAllFromPath($dir."/".$file);
						chmod($dir."/".$file,0777);
						rmdir($dir."/".$file); //remove this directory
					}
					else   
					{
						chmod($dir."/".$file,0777);
						unlink($dir."/".$file);// remove this file
					}
				}
			}
			@closedir($handle);
			return true;
		}
		//
		// end function
		//
		//////////////////////////////////////////////
		// delete category
		if(!deleteAllFromPath($toDeletePath))
		{
			$this->setErrorMessage($lang_script["tmpl.assessment.inc.php"]["ErrorMessage1"]);
			return false;
		}
		// delete calling dir
		if(!@rmdir($toDeletePath))
		{
			$this->setErrorMessage($lang_script["tmpl.assessment.inc.php"]["ErrorMessage2"]);
			return false;
		}
		
		// get ids of categories to delete from db 
		$childIdArray[] = $categoryId;
		$childIdArray = $this->getChildIdArrayFromDb($categoryId,$childIdArray);
		// delete cats from id
		foreach($childIdArray as $catIdToDelete)
		{
	        // delete category
	        $sql = "delete 
				from $this->tmplTabCategory 
	            where CATEGORY_ID_PK = $catIdToDelete";
	        //
			$result = $this->db->query($sql);
	        if (DB::isError($result)) 
	        {
	          $this->setErrorMessage($result->getMessage());
	          continue;
	        }
            
            // delete category
	        $sql = "delete 
				from $this->tmplTabContent 
	            where CATEGORY_ID_PK = $catIdToDelete";
	        //
			$result = $this->db->query($sql);
	        if (DB::isError($result)) 
	        {
	          $this->setErrorMessage($result->getMessage());
	          continue;
	        }
        }
        return true;
    }

    /**
    *	recursive function to get childs of category from db
    *
    *	@access	public
    *	@param	int   	$categoryId
    *	@param	array   $childIdArray
    *   @return array   array with category ids to delete
    */
	function getChildIdArrayFromDb($categoryId,&$childIdArray)
	{
		// if has childs 
		if($this->isNode($categoryId)) 
		{
			// get childs if exist
			$childGroupsData = $this->getChilds($categoryId);
			if(!is_array($childGroupsData))
			{
				return $childIdArray;
			}
			// get childs of childs
			for($i = 0; $i < count($childGroupsData); $i++)
			{
				$childId = $childGroupsData[$i]["CATEGORY_ID_PK"];
				$childIdArray[] = $childId;
				$this->getChildIdArrayFromDb($childId,$childIdArray);
			}
		}
		return $childIdArray;
	}
	
    /**
    *	creates the outputdir for template categories
    *
    *	@access	public
    *	@param	array   $categoryArray
    *   @return true/false
    */
    function makeTmplCategoryDir($categoryArray)
    {
        // extract submitted values
        $name 		= $categoryArray["name"];
        $parentId 	= $categoryArray["parentId"];
        // check, if all nedded values have been submitted
        if($name == "" or $parentId == "")
        {   
            $this->setErrorMessage("makeTmplCategoryDir(): no value for name or parentId submitted");
            return false;
        }
        
        // get parent array
        $parentArray = Array();
        $parentInfo = $this->getParentInfoRecursive($parentId, $parentArray);
        
        //build path
        $reverseParentInfo = array_reverse($parentInfo);
        for($j=0;$j<count($reverseParentInfo);$j++)
        {
            $categoryPath .= "/".$reverseParentInfo[$j]["CATEGORY_NAME"];
        }
        $path = $this->pathToOutputTmpl.$categoryPath."/".$name;
        
        // make dir 
        if (!file_exists($path)) 
        { 
            if( !@mkdir($path, 0777) )
            {
               $this->setErrorMessage("makeTmplCategoryDir(): can't create $path");
               return false;
            }
        } 
               
        return true;
    }
	
    /**
    *	returns info of parent categories
    *
    *	@access	public
    *	@param	int     $parentId
    *   @return array   array with category information
    */
    function getParentInfoRecursive($parentId, &$parentArray)
    {
        $sql = "select CATEGORY_ID_PK, CATEGORY_NAME, CATEGORY_PARENT_ID, CATEGORY_TIME 
			from $this->tmplTabCategory 
            where CATEGORY_ID_PK = $parentId";
        //
        $result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
           $this->setErrorMessage("getParentInfoRecursive(): ".$result->getMessage());
           return false;
        }
        while ($row = $result->fetchRow(DB_FETCHMODE_ASSOC)) 
        {
            $parentArray[] = $row;
		}
        
        $num = count($parentArray);
        
        if($parentArray[$num-1]["CATEGORY_PARENT_ID"] != 0)
        {
            $this->getParentInfoRecursive($parentArray[$num-1]["CATEGORY_PARENT_ID"], $parentArray);
        }
        
        return $parentArray;
    }
    
    /**
    *	returns info to this category
    *
    *	@access	public
    *	@param	int     $categoryId
    *   @return array   array with category information
    */
    function getGroupInfo($categoryId)
    {
        // check if all values are submitted
        if($categoryId == "")
        {   
            $this->setErrorMessage("no value for categoryId submitted");
            return false;
        }
        
        // check if category exists
        if(!$this->isCategory($categoryId))
        {
            return false;
        }
        // get category info
        $sql = "select CATEGORY_ID_PK, CATEGORY_NAME, CATEGORY_PARENT_ID, CATEGORY_TIME 
			from $this->tmplTabCategory 
            where CATEGORY_ID_PK = $categoryId";
		//
        $result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
           $this->setErrorMessage($result->getMessage());
           return false;
        }
        // get table info
		while ($row = $result->fetchRow(DB_FETCHMODE_ASSOC)) 
        {
            foreach($row as $key => $value)
			{
				$categoryArray[$key] = $value;
			}
		}
        
        return $categoryArray;
    }

    /**
    *	get all categories data for overview
    *
    *	@access	public
    *	@param	int		$actualNum
    *	@param	int		$maxRow 
    *   @param  string  $orderby
	*   @param  string  $direction
    *	@param	array	array with all cattegorydata
    */
    function getGroupsForOverview($categoryId, $actualNum = 0, $maxRow, $orderby, $direction)
    {
        //
        // check input
        if($categoryId == "")
        {
            // fetch all userdata
            $sql = "select CATEGORY_ID_PK, CATEGORY_NAME, CATEGORY_PARENT_ID, CATEGORY_TIME 
                from $this->tmplTabCategory
				where CATEGORY_PARENT_ID = 0 
                order by $orderby $direction 
                limit $actualNum, $maxRow"; 
        }
        else
        {
            // fetch all userdata 
            $sql = "select CATEGORY_ID_PK, CATEGORY_NAME, CATEGORY_PARENT_ID, CATEGORY_TIME 
                from $this->tmplTabCategory 
                where CATEGORY_PARENT_ID = $categoryId 
                order by $orderby $direction 
                limit $actualNum, $maxRow"; 
        }
		//
		$result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
            $this->setErrorMessage($result->getMessage());
            return false;
        }
        $k = 0;
        while ($row = $result->fetchRow(DB_FETCHMODE_ASSOC))
        {
            $categoryArray[$k] = $row;
            $k++;
        }
        return $categoryArray;
    }
	
    /**
    *	returns number of categories in this category
    *
    *   if no categoryId is submitted function returns number of all categories   
    *
    *	@access	public
    *   $param  int     $categoryId
    *   @return int     number of categories
    */
    function getGroupsNum($categoryId)
    {
        //
        if($categoryId != "")
        {
            $sql = "select 1 
				from $this->tmplTabCategory 
                where CATEGORY_PARENT_ID = $categoryId";
        }
        else
        {
            $sql = "select 1 
				from $this->tmplTabCategory 
				where CATEGORY_PARENT_ID = 0";
        }
        $result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
            $this->setErrorMessage($result->getMessage());
            return false;
        }
        return $result->numRows();
    }
	
	
	/**
    *	returns number of categories in this category
    *
    *   if no categoryId is submitted function returns number of all categories   
    *
    *	@access	public
    *   $param  int     $categoryId
    *   @return int     number of content
    */
    function getContentNum($categoryId)
    {
        //
        if($categoryId != "")
        {
            $sql = "select 1 
				from $this->tmplTabContent 
                where CONTENT_CATEGORY_ID_FK = $categoryId";
        }
        else
        {
            $sql = "select 1 
				from $this->tmplTabContent 
				where CONTENT_CATEGORY_ID_FK = 0";
        }
        $result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
            $this->setErrorMessage($result->getMessage());
            return false;
        }
        return $result->numRows();
    }
									 
    /**
    *	get all content data for overview
    *
    *	@access	public
	*
	*	@param	int		$categoryId
    *	@param	string	$actualNum
    *	@param	string	$maxRow 
    *   @param  string  $orderby
	*   @param  string  $direction
    *	@param	array	array with all groupdata
    */
    function getContentForOverview($categoryId, $actualNum = 0, $maxRow, $orderby, $direction)
    {
        global $cfgTabAuthContentToCategory;
        
        // check input
        if($categoryId == "")
        {
            // fetch all userdata
            $sql = "select CONTENT_ID_PK, CONTENT_CATEGORY_ID_FK, CONTENT_NAME, CONTENT_FILENAME, CONTENT_ENDING, CONTENT_SIZE, CONTENT_TIME 
                from $this->tmplTabContent 
                order by $orderby $direction 
                limit $actualNum, $maxRow"; 
        }
        else
        {
            // fetch all userdata
            $sql = "select CONTENT_ID_PK, CONTENT_CATEGORY_ID_FK, CONTENT_NAME, CONTENT_FILENAME, CONTENT_ENDING, CONTENT_SIZE, CONTENT_TIME 
                from $this->tmplTabContent 
                where CONTENT_CATEGORY_ID_FK = $categoryId 
                order by $orderby $direction 
                limit $actualNum, $maxRow"; 
        }

        $result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
            $this->setErrorMessage($sql." ".$result->getMessage());
			return false;
        }
		$k = 0;
        while ($row = $result->fetchRow(DB_FETCHMODE_ASSOC)) 
        {
        	$contentArray[$k] = $row;
			$k++;
        }
        //
	    return $contentArray;
    }
    
    /**
    *	add Content To Category
    *
    *	@access	public
    *	@param	string	$filesName
    *	@param	int		$amountFiles
    *   @param  int		$categoryId
    *	@param	array	array with all groupdata
    */
    function addContentToCategory($filesName,$amountFiles,$categoryId=0)
    {
	    global $lang_script;
		// get path
		$parentInfo = array();
		$parentInfo = $this->getParentInfoRecursive($categoryId, &$data);
			
		if (!$parentInfo)
		{
			$this->setErrorMessage("No path in Function:  addContentToCategory");
			$uploadData[0]["error"] = $lang_script["tmpl.assessment.inc.php"]["uploadData0Error"];
			return $uploadData;
		}
        //build path
        $reverseParentInfo = array_reverse($parentInfo);
        for($j=0;$j<count($reverseParentInfo);$j++)
        {
            $categoryPath .= "/".$reverseParentInfo[$j]["CATEGORY_NAME"];
        }
		// complete path
        $completePath = $this->pathToOutputTmpl.$categoryPath;

		// init upload class
		$fileFunct	= &new tmplUploadFunctions;
		// upload files
		for($i = 0;$i < $amountFiles;$i++)
		{
			// upload file
			$uploadData[$i] = $fileFunct->tmplUploadFiles($completePath, $filesName.$i);
		}
		//
		// define error messages
		for($i = 0; $i < $amountFiles; $i++)
		{
			// valid file
			if ($uploadData[$i]["error"])
			{
				if ($uploadData[$i]["error"] == $lang_script["tmpl.assessment.inc.php"]["uploadData1error"])
				{
					$uploadData[$i]["error"] = $lang_script["tmpl.assessment.inc.php"]["uploadData2error"];
				}
				else 
				{
					$uploadData[$i]["error"] = $lang_script["tmpl.assessment.inc.php"]["uploadData3error"];
				}
			}
			$uploadData[$i]["time"] = date("y-m-d H:i:s");    //030116-rr scheinbar Rest aus Test
			$uploadData[$i]["path"] = $categoryPath;
		}
		//
		// save data to db
		for($i = 0; $i < count($uploadData); $i++)
		{
			if($uploadData[$i]["error"])
			{
				continue;
			}
			// check for existing entry
	        $sql = "select CONTENT_ID_PK 
				from $this->tmplTabContent 
				where CONTENT_FILENAME = '".$uploadData[$i]['name']."' 
				and CONTENT_CATEGORY_ID_FK = $categoryId";
			//
			$result = $this->db->query($sql);
			if (DB::isError($result)) 
	        {
	           $this->setErrorMessage($result->getMessage());
	           $uploadData[$i]["error"] = $lang_script["tmpl.assessment.inc.php"]["uploadData4error"];
			   continue;
	        }
			// get id if exists
			$row = $result->fetchRow(DB_FETCHMODE_ASSOC);
			// if result else nextid
			if(!($nextId = $row["CONTENT_ID_PK"]))
			{
				// get next free id
		        $nextId = $this->db->nextId($this->tmplTabContent);
		        if (DB::isError($nextId)) 
		        {
		            $this->setErrorMessage($result->getMessage());
					$uploadData[$i]["error"] = $lang_script["tmpl.assessment.inc.php"]["uploadData4error"];
					continue;
		        }
		        // create insert statement
		        $sql = "insert 
					into $this->tmplTabContent 
		            (CONTENT_ID_PK, CONTENT_CATEGORY_ID_FK, CONTENT_NAME, CONTENT_FILENAME, CONTENT_ENDING, CONTENT_SIZE, CONTENT_TIME) 
		            values 
		            ($nextId, $categoryId, '".$uploadData[$i]['real']."', '".$uploadData[$i]['name']."', '".$uploadData[$i]['ext']."', '".$uploadData[$i]['size']."', NOW())";
				//
		        $result = $this->db->query($sql);
		        if (DB::isError($result)) 
		        {
		           $this->setErrorMessage($result->getMessage());
				   $uploadData[$i]["error"] = $lang_script["tmpl.assessment.inc.php"]["uploadData4error"];
				   continue;
		        }
			}
			else
			{
		        // create update statement
		        $sql = "update $this->tmplTabContent 
						set 
		        		CONTENT_NAME = '".$uploadData[$i]['real']."', 
						CONTENT_SIZE = '".$uploadData[$i]['size']."', 
						CONTENT_TIME = NOW() 
						where CONTENT_ID_PK = $nextId";
				//
		        $result = $this->db->query($sql);
		        if (DB::isError($result)) 
		        {
		           $this->setErrorMessage($result->getMessage());
				   $uploadData[$i]["error"] = $lang_script["tmpl.assessment.inc.php"]["uploadData4error"];
				   continue;
		        }
			}
		}
		//
		return $uploadData;
    }

    /**
    *	deletes all content of this category
    *
    *	@access	public
    *	@param	int $contentId
    *   @return string true/false
    */
    function deleteContent($contentId)
    {
	   global $lang_script;
		// get file data
		$contentData = $this->getContentData($contentId);
		// delete file from directory
		if(!@unlink($this->pathToOutputTmpl.$contentData["path"]."/".$contentData["CONTENT_FILENAME"]))
		{
           $this->setErrorMessage($lang_script["tmpl.assessment.inc.php"]["uploadData5error"]);
           return false;
		}
		// delete content from database
		$sql = "delete 
			from $this->tmplTabContent 
			where CONTENT_ID_PK = $contentId";
        //
        $result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
           $this->setErrorMessage($result->getMessage());
           return false;
        }
        return true;
    }

    /**
    *	gets content data from database
    *
    *	@access	private
    *	@param	int $contentId
    *   @return array true/false
    */
    function getContentData($contentId)
    {
	    // check if all values are submitted
        if($contentId == "")
        {   
            //$this->setErrorMessage("no value for contentId submitted");
            return false;
        }
        // get data from database
        $sql = "select CONTENT_ID_PK, CONTENT_CATEGORY_ID_FK, CONTENT_NAME, CONTENT_FILENAME, CONTENT_ENDING, CONTENT_SIZE, CONTENT_TIME 
			from $this->tmplTabContent 
            where CONTENT_ID_PK = $contentId";
        //
		$result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
           $this->setErrorMessage($result->getMessage());
           return false;
        }
        while ($row = $result->fetchRow(DB_FETCHMODE_ASSOC)) 
        {
            $contentArray = $row;
        }
		// get category id		
		$categoryId = $contentArray["CONTENT_CATEGORY_ID_FK"];
        // get path
		$parentInfo = array();
		$parentInfo = $this->getParentInfoRecursive($categoryId, &$data);
		//	
		if (!$parentInfo)
		{
			$this->setErrorMessage("No path in Function:  getContentData");
			return false;
		}
        //build path
        $reverseParentInfo = array_reverse($parentInfo);
        for($j=0;$j<count($reverseParentInfo);$j++)
        {
            $categoryPath .= "/".$reverseParentInfo[$j]["CATEGORY_NAME"];
        }
		// add complete path to content Array
		$contentArray["path"] = $categoryPath;
		//
        return $contentArray;
    }

	/**
    *	gets all Pathes
    *
    *	@access	private
    *   @return array path with Id as $key
    */
    function getAllTemplatePathes()
    {
	global $lang_script;
        // get category info
        $sql = "select CATEGORY_ID_PK, CATEGORY_NAME 
			from $this->tmplTabCategory 
            where not CATEGORY_ID_PK = 0 
			order by CATEGORY_NAME";
		//
        $result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
           $this->setErrorMessage($result->getMessage());
           return false;
        }
        //
		while ($row = $result->fetchRow(DB_FETCHMODE_ASSOC)) 
        {
            foreach($row as $key => $value)
			{
				$categoryArray[$key] = $value;
			}
			$allCategoryArray[] = $categoryArray;
		}
		// get path and id
		for($i = 0; $i < count($allCategoryArray);$i++)
		{
			$categoryData[$i]["catId"]		=	$categoryId		=	$allCategoryArray[$i]["CATEGORY_ID_PK"];
			// get path
			$parentInfo = array();
			$parentInfo = $this->getParentInfoRecursive($categoryId, &$parentInfo);
			//
			if (!$parentInfo)
			{
				$this->setErrorMessage("No path in Function:  addContentToCategory");
				$uploadData[0]["error"] = $lang_script["tmpl.assessment.inc.php"]["uploadData0error"];
				return false;
			}
	        //build path
	        $reverseParentInfo = array_reverse($parentInfo);
	        for($j=0;$j<count($reverseParentInfo);$j++)
	        {
	            $categoryPath .= "/".$reverseParentInfo[$j]["CATEGORY_NAME"];
	        }
			// add path
			$categoryData[$i]["catPath"]	=	$categoryPath;
			// free data
			$categoryPath = "";
		}
		//
		return $categoryData;
	}
	
    /**
    *	checks if category exists
    *
    *	@access	public
    *	@param	string	$userid
    *   @return string true/false
    */
    function isCategory($categoryId)
    {
         // check if user exists
        $sql = "select 1 
			from $this->tmplTabCategory 
			where CATEGORY_ID_PK = $categoryId";
		//
		$result = $this->db->query($sql);
        if($result->numRows() != 1)
        {   
            $this->setErrorMessage("category with this id does not exist");
            return false;
        }
        
        return true;
    }
   
    /**
    *	checks if category does exist with same name
    *
    *	@access	public
    *	@param	string	$catName
    *	@param	string	$parentId
    *   @return string  true/false
    */
    function existsCategory($catName, $parentId)
    {
        // check if all values are submitted
        if($catName == "" or $parentId == "")
        {   
            $this->setErrorMessage("no value for catName or parentId submitted");
            return false;
        }
        
        // check if category exists
        $sql = "select 1 
			from $this->tmplTabCategory 
        	where CATEGORY_NAME = '$catName' 
			and CATEGORY_PARENT_ID = $parentId";
		//
        $result = $this->db->query($sql);
        if($result->numRows() == 1)
        {   
            $this->setErrorMessage("the category with this name and parent already exists");
            return false;
        }
        
        return true;
    }
    
    /**
    *	checks if node has childs
    *
    *	@access	public
     *	@param	int     $categoryId 
    *   @return string  true/false
    */
    function isNode($categoryId)
    {
        //
        $sql = "select 1 
			from $this->tmplTabCategory 
			where CATEGORY_PARENT_ID = $categoryId";
        //
        $result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
            return $this->db->getMessage();
        }
        
        if ($result->numRows()  > 0)
        {
            return true;
        }
                
        return  false;
    }
    
    /**
    *	checks if node has childs and returns array with childs
    *
    *	@access	public
     *	@param	int     $id   group id
    *   @return array  array with child data
    */
    function getChilds($id)
    {
    	//
        $sql = "select CATEGORY_ID_PK 
			from $this->tmplTabCategory 
			where CATEGORY_PARENT_ID = $id";
        //
        $result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
            return $this->db->getMessage();
        }
        
        while ($row = $result->fetchRow(DB_FETCHMODE_ASSOC))
        {
            $childArray[] = $row;
        }
                
        return  $childArray;
    }
	
	
    /**
    *	reads code from file
    *
    *	@access	public
     *	@param	int     $id   group id
    *   @return array  array with child data
    */
	function readFileCode($filePath,$fileName,$phpSave=false)
	{
		global $lang_script;
		// Template auslesen
		$fileTemplate	= @fopen($filePath."/".$fileName,"r");
		if(!$fileTemplate)
		{
			$this->setErrorMessage("File :".$fileName." ".$lang_script["tmpl.assessment.inc.php"]["ErrorMessage3"]);
			return false;
		}
		$fileContent	= @fread($fileTemplate,1000000);
		if(!$fileContent)
		{
			$this->setErrorMessage("File :".$fileName." ".$lang_script["tmpl.assessment.inc.php"]["ErrorMessage4"]);
			return false;
		}
		// close
		fclose($fileTemplate);
		// clean data
		if($phpSave)
		{
			$fileContent	=	$this->checkForInvalidExpressions($fileContent);
		}
		// return data
		return $fileContent;
	}
	
    /**
    *	checks for invald expressions
    *
    *	@access	public
    *	@param	string     $fileContent
    *   @return string/false
    */
	function checkForInvalidExpressions($fileContent)
	{
		if (ereg('<\?', $fileContent))
		{
			return false;
		}
		// return data
		return $fileContent;
	} 
	
		
} // end of class catFunctions
?>
