<?php
// +----------------------------------------------------------------------+
// | sa auth module - user management xmlrpc server                       |
// +----------------------------------------------------------------------+
// | Copyright (c) 2002 serie a - Digitale Medien und Systeme GmbH        |
// +----------------------------------------------------------------------+
// | Authors: Jrn Allert <allert@serie-a.de>                             |
// |          Rene Renk <renk@serie-a.de>                                 |
// +----------------------------------------------------------------------+
//
// $Id: tmpl.tree.inc.php,v 1.1.1.1 2004/07/15 13:34:55 opensurveypilot Exp $

// start session
@session_start();
 
// require all configuration files and libraries
require_once("./../config/config.inc.php"); // common config file
require_once($cfgPathToAuthFiles."/config/auth.file.inc.php"); // auth files
require_once($cfgPathToSurveyFiles."/config/survey.config.inc.php"); // survey configuration
require_once($cfgPathToSurveyFiles."/config/survey.file.inc.php"); // survey files
require_once($cfgPathToTmplFiles."/config/tmpl.file.inc.php"); // survey files
require_once($cfgPathToAuthFiles."/lib/permission.inc.php"); // permission functions
require_once ("DB.php"); // PEAR::DB

/**
*	Class to create Javascript which in turn creates the project tree (which is not really a tree)
*
*	@author otto <otto@poellath.org>
*	@see PEAR::DB
*/
class TmplTreeMenu
{
	/**
	* Name of the image used as node icon
	* @var 		string $icon
	* @access 	private
	*/
    var $icon = "folder.gif";
	
	/**
	* Name of the image used as modul-node icon
	* @var 		string $modulIcon
	* @access 	private
	*/
    var $modulIcon = "designfolder.gif";

	/**
	* The menu object
	* @var 		object HTML_TreeMenu $menu
	* @access 	private
	*/
    var $menu;

	/**
	* The DB handle
	* @var 		mixed $db
	* @see		DB::connect()
	* @access 	private
	*/
    var $db = "";

	/**
	* Name the DB table node elements are read from
	* @var 		string $table
	* @access 	private
	*/
    var $table;
    
	/**
	* The time when tree building is started
	* @var 		int $startTime
	* @access 	private
	*/
    var $startTime;

	/**
	* The time in seconds after which tree building is aborted
	* @var 		int $timeout
	* @access 	private
	*/
    var $timeout = 10;

	/**
	* The last error message
	* @var 		string $error;
	* @access 	private
	*/
    var $error;
    
	/**
	*	Constructor
	*
	*	@access public
	*	@param string $table 	Name the DB table node elements are read from
	*/
    function TmplTreeMenu()
    {
        $this->db= DB::connect(DSN, true);
        if (DB::isError($this->db)) 
        {    
           return false;
        }
        
        $this->buildTree();
    }
    
	/**
	*	Check for an error since last invocation of buildTree()
	*
	*	@access public
	*	@return boolean     TRUE if a timeout occurred, FALSE otherwise
	*/
    function isError()
    {
    	return ($this->error != "");
    }
    
	/**
	*	Get the error message
	*
	*	@access public
	*	@return string      Error message
	*/
    function getError()
    {
    	return $this->error;
    }
    
	/**
	*	Build the user tree
	*
	*	@access private
	*/
    function buildTree()
    {
	    global $lang_script;
    	$this->error = "";
        $this->startTime = time();
        $this->menu = new HTML_TreeMenu("menuLayer", '../media/images/treemenu', '_self');
        
    	//$rootNode = new HTML_TreeNode("Templatebersicht", $this->getLinkForRootNode(), "rootfolder.gif");		
		$rootNode = new HTML_TreeNode($lang_script["tmpl.tree.inc.php"]["rootNodeTemplate"], "#", "rootfolder.gif");		
		$surveyNode = new HTML_TreeNode($lang_script["tmpl.tree.inc.php"]["surveyNodeTemplate"], $this->getLinkForSurvey(), $this->modulIcon);
		$assessmentNode = new HTML_TreeNode($lang_script["tmpl.tree.inc.php"]["assessmentNodeTemplate"], $this->getLinkForAssessment(), $this->modulIcon);
		
		$this->menu->addItem($rootNode);
		$rootNode->addItem($surveyNode);
		$rootNode->addItem($assessmentNode);
		
    	$this->appendChildrenToNode_survey($surveyNode, 0);
		$this->appendChildrenToNode_assessment($assessmentNode, 0);

    }
    
	/**
	*	Append all existing child nodes to a node
	*
	*	@access private
	*	@param object HTML_TreeNode &$node		Node to append children to
	*	@param string $nodeId					The node's DB-ID
	*/
    function appendChildrenToNode_survey(&$node, $nodeId)
    {
    	if(time() - $this->startTime > $this->timeout)
        { 
			$this->error = "Timeout. Tree build aborted after ".(time() - $this->startTime)." seconds.";
    		return false;
    	}
    
    	$children = $this->getChildrenForNodeWithId_survey($nodeId);
    	if($children)
        {
			for($i=0; $i<count($children); $i++)
            {
				$node->addItem($children[$i][1]);
				$this->appendChildrenToNode_survey($children[$i][1], $children[$i][0]);
			}
    	}
    }
	
	/**
	*	Append all existing child nodes to a node
	*
	*	@access private
	*	@param object HTML_TreeNode &$node		Node to append children to
	*	@param string $nodeId					The node's DB-ID
	*/
    function appendChildrenToNode_assessment(&$node, $nodeId)
    {
    	if(time() - $this->startTime > $this->timeout)
        { 
			$this->error = "Timeout. Tree build aborted after ".(time() - $this->startTime)." seconds.";
    		return false;
    	}
    
    	$children = $this->getChildrenForNodeWithId_assessment($nodeId);
    	if($children)
        {
			for($i=0; $i<count($children); $i++)
            {
				$node->addItem($children[$i][1]);
				$this->appendChildrenToNode_assessment($children[$i][1], $children[$i][0]);
			}
    	}
    }
    
	/**
	*	Holt alle Kinder eines Knotens aus der DB
	*
	*	@access private
	*	@param string $id						The node's DB ID
	*	@return mixed							FALSE if no children are found, otherwise an array with 1 element for each child found,
	* 											each consisting of an array containing the child node's DB ID and an object of type HTML_TreeNode
	* 											representing the child node.
	*/
    function getChildrenForNodeWithId_survey($id)
    {
		global $cfgTmplTabCategory;
		
    	$children = false;
    	
        $sql = "select CATEGORY_ID_PK, CATEGORY_NAME 
			from $cfgTmplTabCategory 
			where CATEGORY_PARENT_ID = '$id' 
			order by CATEGORY_NAME";
        //
        $result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
            return "Fehler. ".$this->db->getMessage();
        }
        
        if ($result->numRows() == 0)
        {
            return false;
        }
        
        // fetch userdata
        while ($row = $result->fetchRow(DB_FETCHMODE_ASSOC)) 
        {
            $children[] = array($row['CATEGORY_ID_PK'], new HTML_TreeNode($row['CATEGORY_NAME'], $this->getLinkForId_survey($row['CATEGORY_ID_PK']), $this->icon));
        }
    	
    	return $children;
    }
	
	/**
	*	Holt alle Kinder eines Knotens aus der DB
	*
	*	@access private
	*	@param string $id						The node's DB ID
	*	@return mixed							FALSE if no children are found, otherwise an array with 1 element for each child found,
	* 											each consisting of an array containing the child node's DB ID and an object of type HTML_TreeNode
	* 											representing the child node.
	*/
    function getChildrenForNodeWithId_assessment($id)
    {
		global $cfgTabAssessmentCategory;
		
    	$children = false;
    	
        $sql = "select CATEGORY_ID_PK, CATEGORY_NAME 
			from $cfgTabAssessmentCategory 
			where CATEGORY_PARENT_ID = '$id' 
			order by CATEGORY_NAME";
        //
        $result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
            return "Fehler. ".$this->db->getMessage();
        }
        
        if ($result->numRows() == 0)
        {
            return false;
        }
        
        // fetch userdata
        while ($row = $result->fetchRow(DB_FETCHMODE_ASSOC)) 
        {
            $children[] = array($row['CATEGORY_ID_PK'], new HTML_TreeNode($row['CATEGORY_NAME'], $this->getLinkForId_assessment($row['CATEGORY_ID_PK']), $this->icon));
        }
    	
    	return $children;
    }
	
	
	/**
	*	Creates the URL to link a node to
	*
	*	@access private
	*	@param string $id						The node's DB ID
	*/
    function getLinkForId_survey($id)
    {
		global $parent_id;
    	global $fileTmplToplevel, $fileFrameToplevelName;
        global $fileTmplContentOverview, $fileFrameContentName;
        
    	return "javascript:changeTwoFrames(\'$fileTmplToplevel?parentId=$id&module=sur\',\'$fileFrameToplevelName\',\'$fileTmplContentOverview?groupId=$id&module=sur\',\'$fileFrameContentName\')";
    }
	
	/**
	*	Creates the URL to link a node to
	*
	*	@access private
	*	@param string $id						The node's DB ID
	*/
    function getLinkForId_assessment($id)
    {
		global $parent_id;
    	global $fileTmplToplevel, $fileFrameToplevelName;
        global $fileTmplContentOverview, $fileFrameContentName;
        
    	return "javascript:changeTwoFrames(\'$fileTmplToplevel?parentId=$id&module=ass\',\'$fileFrameToplevelName\',\'$fileTmplContentOverview?groupId=$id&module=ass\',\'$fileFrameContentName\')";
    }
	
	function getLinkForSurvey()
    {
    	global $fileTmplToplevel, $fileFrameToplevelName;
    	global $fileTmplOverview, $fileFrameContentName;
		
    	return "javascript:changeTwoFrames(\'$fileTmplToplevel?module=sur\',\'$fileFrameToplevelName\',\'$fileTmplOverview?module=sur\',\'$fileFrameContentName\')";

    }
	
	function getLinkForAssessment()
    {
    	global $fileTmplToplevel, $fileFrameToplevelName;
    	global $fileTmplOverview, $fileFrameContentName;
		
    	return "javascript:changeTwoFrames(\'$fileTmplToplevel?module=ass\',\'$fileFrameToplevelName\',\'$fileTmplOverview?module=ass\',\'$fileFrameContentName\')";
    }
	
    function getLinkForRootNode()
    {
    	global $fileTmplToplevel, $fileFrameToplevelName;
    	global $fileTmplOverview, $fileFrameContentName;
		
    	return "javascript:changeTwoFrames(\'$fileTmplToplevel\',\'$fileFrameToplevelName\',\'$fileTmplOverview\',\'$fileFrameContentName\')";
    }
	
	/**
	*	Echos the Javascript code to create the user tree
	*
	*	@access public
	*/
	function printMenu()
    {
		$this->menu->printMenu();
	}
    
} // end of class CategoryTreeMenu
?>
