<?php
// +----------------------------------------------------------------------+
// | sa auth module - category management                                 |
// +----------------------------------------------------------------------+
// | Copyright (c) 2002 serie a - Digitale Medien und Systeme GmbH        |
// +----------------------------------------------------------------------+
// | Authors: Jrn Allert <allert@serie-a.de>                             |
// |          Rene Renk <renk@serie-a.de>                                 |
// +----------------------------------------------------------------------+
// | Description:                                                         |
// +----------------------------------------------------------------------+
//
// $Id: category.inc.php,v 1.1.1.1 2004/07/15 13:35:16 opensurveypilot Exp $

// require all configuration files and libraries
require_once($cfgPathToProjectAdmin."/config/config.inc.php"); // common config file
require_once($cfgPathToConf."/file.inc.php"); // project files

require_once ("DB.php"); // PEAR::DB
require_once ("Validate.php"); // PEAR::Validate

/**
* categoryFunctions class
*
* Category Management Class
*
* @author  Jrn Allert <allert@serie-a.de>
* @author  Rene Renk <renk@serie-a.de>
* @access  public
*/
class categoryFunctions
{
    /**
    *	handle for db connection
    *	@var    string  $db
    */
    var $db = "";
    
    /**
    *	error message
    *	@var    string  $errorMessage
    */
    var $errorMessage;
    
    /**
    *	create new catFunctions object 
    *
    *	@access	public
    */
    function categoryFunctions()
    {
        $this->errorMessage = "";
        // connect to db
        $this->db = DB::connect(DSN, true);
        if (DB::isError($db)) 
        {    
            $this->setErrorMessage($db->getMessage());
            return false;
        }
    }
    
    /**
    *	sets error message
    *
    *	@access	private
    *	@param	string	$err
    */
    function setErrorMessage($err)
    {
        $this->errorMessage = $err;
    }
    
    /**
    *	adds a category
    *
    *	@access	public
    *	@param	array	$catArray
    *   @return int $nextId id of the new group
    */
    function addCategory($catArray)
    {
        global $cfgTabAuthCategory;
        
        // extract submitted values
        $name = $catArray["name"];
        $parentId = $catArray["parentId"];
        // check,if all nedded values have been submitted
        if($name == "" or $parentId == "")
        {   
            $this->setErrorMessage("no value for name or parentId submitted");
            return false;
        }
        
        // check if this category already exists
        if(!$this->existsCategory($name, $parentId))
        {
            return false;
        }
        
        
        // check if parentId exists
        if($parentId != "0" and $parentId != "")
        {
            if(!$this->isCategory($parentId))
            {
                $this->setErrorMessage("There is no parent category with this id.");
                return false;
            }
        }
        
        // get next free id
        $nextId = $this->db->nextId($cfgTabAuthCategory);
        if (DB::isError($nextId)) 
        {
            $this->setErrorMessage($result->getMessage());
            return false;
        }
        
	    // create insert statement
        $sql = "insert into $cfgTabAuthCategory 
            (CATEGORY_ID_PK, CATEGORY_NAME, CATEGORY_PARENT_ID, CATEGORY_CATEGORYTIME) 
            values 
            ('$nextId', '$name', '$parentId', NOW())";
        
        $result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
           $this->setErrorMessage($result->getMessage());
           return false;
        }
        
        return $nextId;
    }
    
    /**
    *	deletes a category
    *
    *   use emptyCategory($catId), if you want to delete all content in this category
    *
    *	@access	public
    *	@param	int $catId
    *   @return string true/false
    */
    function deleteCategory($catId)
    {
        global $cfgTabAuthCategory;
        
        // check if all values are submitted
        if($catId == "")
        {   
            $this->setErrorMessage("no value for catId submitted");
            return false;
        }
        
        // check if categiory exists
        if(!$this->isCategory($catId))
        {
            return false;
        } 
        
        // delete category
        $sql = "delete 
			from $cfgTabAuthCategory  
            where CATEGORY_ID_PK = $catId";
		//
        $result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
           $this->setErrorMessage($result->getMessage());
           return false;
        }
        
        return true;
    }
    
    /**
    *	returns info to this category
    *
    *	@access	public
    *	@param	int     $categoryId
    *   @return array   array with group information
    */
    function getCategoryInfo($categoryId)
    {
        global $cfgTabAuthCategory;
        
        // check if all values are submitted
        if($categoryId == "")
        {   
            $this->setErrorMessage("no value for categoryId submitted");
            return false;
        }
        
        // check if category exists
        if(!$this->isCategory($categoryId))
        {
            return false;
        }
        
        // get category info
        $sql = "select * 
			from $cfgTabAuthCategory  
            where CATEGORY_ID_PK = '$categoryId'";
		//
        $result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
           $this->setErrorMessage($result->getMessage());
           return false;
        }
        
        // get num rows for groups in this category
        $categoryArray["groupNum"] = $this->getGroupNum($categoryId);
        
        // get num rows for content in this category
        $categoryArray["contentNum"] = $this->getContentNum($categoryId);
        
        // get num rows for categories in this category
        $categoryArray["catNum"] = $this->getCategoriesNum($parentId);
		
        // get table info
		while ($row = $result->fetchRow(DB_FETCHMODE_ASSOC)) 
        {
            foreach($row as $key => $value)
			{
				$categoryArray[$key] = $value;
			}
		}
        
		//print_r($categoryArray);
        return $categoryArray;
    }

    /**
    *	get all categories data for overview
    *
    *	@access	public
    *	@param	string	$actualNum
    *	@param	string	$maxRow 
    *   @param  string  $orderby
    *	@param	array	array with all groupdata
    */
    function getCategoriesForOverview($categoryId, $actualNum = 0, $maxRow, $orderby, $direction)
    {
        global $cfgTabAuthCategory;
        
        // check input
        if($categoryId == "")
        {
            // fetch all userdata
            $sql = "select * 
                from $cfgTabAuthCategory
				where CATEGORY_PARENT_ID = '0' 
                order by $orderby $direction 
                limit $actualNum, $maxRow"; 
        }
        else
        {
            // fetch all userdata 
            $sql = "select *
                from $cfgTabAuthCategory 
                where CATEGORY_PARENT_ID = '$categoryId'
                order by $orderby $direction 
                limit $actualNum, $maxRow"; 
        }

        $result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
            $this->setErrorMessage($result->getMessage());
            return false;
        }
        $k = 0;
        while ($row = $result->fetchRow(DB_FETCHMODE_ASSOC)) 
        {
            $catArray[$k] = $row;
            $k++;
        }
        return $catArray;
    }
	
    /**
    *	returns number of categories in this category
    *
    *   if no categoryId is submitted function returns number of all groups   
    *
    *	@access	public
    *   $param  string  $categoryId
    *   @return int     number of users
    */
    function getCategoriesNum($categoryId = "")
    {
        global $cfgTabAuthCategory;
        
        if($categoryId != "")
        {
            $sql = "select 1 
				from $cfgTabAuthCategory  
                where CATEGORY_PARENT_ID = $categoryId";
        }
        else
        {
            $sql = "select 1 
				from $cfgTabAuthCategory
				where CATEGORY_PARENT_ID = '0'";
        }
        $result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
            $this->setErrorMessage($result->getMessage());
            return false;
        }
        
        return $result->numRows();
    }
	
    /**
    *	returns number of groups in this category
    *
    *   if no categoryId is submitted function returns number of all groups   
    *
    *	@access	public
    *   $param  string  $categoryId
    *   @return int     number of users
    */
    function getGroupNum($categoryId)
    {
        global $cfgTabGroupToCatToPerm;
        global $cfgTabAuthGroup;
        
        if($categoryId != "")
        {
            $sql = "select 1 
				from $cfgTabGroupToCatToPerm  
                where CTGRY_TO_PRMSSN_CTGRY_ID_FK = $categoryId";
        }
        else
        {
            $sql = "select 1 
				from $cfgTabAuthGroup";
        }
        $result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
            $this->setErrorMessage($result->getMessage());
            return false;
        }
        
        return $result->numRows();
    }
    
    /**
    *	returns number of groups in this category
    *
    *   if no categoryId is submitted function returns number of all groups   
    *
    *	@access	public
    *   $param  string  $categoryId
    *   @return int     number of users
    */
    function getContentNum($categoryId)
    {
        global $cfgTabAuthContentToCategory;
        
        if($categoryId != "")
        {
            $sql = "select 1 
				from $cfgTabAuthContentToCategory  
                where CNTNT_TO_CTGRY_CATEGORY_ID_FK = $categoryId";
        }
        else
        {
            $sql = "select 1 
				from $cfgTabAuthContentToCategory";
        }
        $result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
            $this->setErrorMessage($result->getMessage());
            return false;
        }
        
        return $result->numRows();
    }
    
    /**
    *	get all group data for overview
    *
    *	@access	public
    *	@param	string	$actualNum
    *	@param	string	$maxRow 
    *   @param  string  $orderby
    *	@param	array	array with all groupdata
    */
    function getGroupsForOverview($categoryId, $actualNum = 0, $maxRow, $orderby, $direction)
    {
        global $cfgTabAuthGroup;
        global $cfgTabGroupToCatToPerm;
        
        // check input
        if($categoryId == "")
        {
            // fetch all userdata
            $sql = "select * 
                from $cfgTabAuthGroup 
                order by $orderby $direction 
                limit $actualNum, $maxRow"; 
        }
        else
        {
            // fetch all userdata 
            $sql = "select 
                $cfgTabAuthGroup.GROUP_ID_PK as groupId, 
                $cfgTabAuthGroup.GROUP_NAME, 
                $cfgTabAuthGroup.GROUP_PARENT_ID,
		$cfgTabGroupToCatToPerm.CTGRY_TO_PRMSSN_ID_PK, 
                $cfgTabGroupToCatToPerm.CTGRY_TO_PRMSSN_PERMSSN_ID_FK,
		$cfgTabGroupToCatToPerm.CTGRY_TO_PRMSSN_PERMSSNTIME 
                from $cfgTabAuthGroup, $cfgTabGroupToCatToPerm 
                where $cfgTabAuthGroup.GROUP_ID_PK = $cfgTabGroupToCatToPerm.CTGRY_TO_PRMSSN_GROUP_ID_FK and 
                $cfgTabGroupToCatToPerm.CTGRY_TO_PRMSSN_CTGRY_ID_FK = $categoryId 
                order by $orderby $direction 
                limit $actualNum, $maxRow"; 
        }
        
        $result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
            $this->setErrorMessage($result->getMessage());
            return false;
        }
        
        while ($row = $result->fetchRow(DB_FETCHMODE_ASSOC)) 
        {
            $groupArray[] = $row;
        }
        return $groupArray;
    }
    
    /**
    *	get all content data for overview
    *
    *	@access	public
    *	@param	string	$actualNum
    *	@param	string	$maxRow 
    *   @param  string  $orderby
    *	@param	array	array with all groupdata
    */
    function getContentForOverview($categoryId, $actualNum = 0, $maxRow, $orderby, $direction)
    {
        global $cfgTabAuthContentToCategory;
        
        // check input
        if($categoryId == "")
        {
            // fetch all userdata
            $sql = "select * 
                from $cfgTabAuthContentToCategory 
                order by $orderby $direction 
                limit $actualNum, $maxRow"; 
        }
        else
        {
            // fetch all userdata
            $sql = "select * 
                from $cfgTabAuthContentToCategory 
                where CNTNT_TO_CTGRY_CATEGORY_ID_FK = $categoryId 
                order by $orderby $direction 
                limit $actualNum, $maxRow"; 
        }
		//
        $result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
            $this->setErrorMessage($sql." ".$result->getMessage());
            return false;
        }
		$k = 0;
        while ($row = $result->fetchRow(DB_FETCHMODE_ASSOC)) 
        {
        	$groupArray[$k] = $row;
			$k++;
        }
       
	    return $groupArray;
    }
    
    function addContentToCategory($contentTypeId, $contentId, $contentName, $categoryId)
    {
        global $cfgTabAuthContentToCategory;
                
        // check if all values are submitted
        if($contentTypeId == "" or $contentId == "" or $categoryId == "" or $contentName == "")
        {   
            $this->setErrorMessage("no value for contentTypeId or contentId or categoryId or contentName submitted");
            return false;
        }
        
        // check if content already is in tabel content2category
        if(!$this->existsContent($contentTypeId, $contentId, $categoryId))
        {   
            return false;
        }
        
        // get next free id
        $nextId = $this->db->nextId($cfgTabAuthContentToCategory);
        if (DB::isError($nextId)) 
        {
            $this->setErrorMessage($result->getMessage());
            return false;
        }
                
        // insert content
        $sql = "insert into $cfgTabAuthContentToCategory 
            (CNTNT_TO_CTGRY_ID_PK, CNTNT_TO_CTGRY_CATEGORY_ID_FK, CNTNT_TO_CTGRY_CNTNTTYP_ID_FK, CNTNT_TO_CTGRY_CONTENT_ID, CNTNT_TO_CTGRY_CONTENTNAME, CNTNT_TO_CTGRY_CONTENTTIME) 
            values 
            ('$nextId', '$categoryId', '$contentTypeId', '$contentId', '$contentName', NOW())";
        //
		$result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
           $this->setErrorMessage($result->getMessage());
           return false;
        }
        
        return true;
    }
    
    function deleteContentFromCategory($contentTypeId, $contentId, $categoryId)
    {
        global $cfgTabAuthContentToCategory;
        
        if($contentId != "")
        {
            // delete content
            $sql = "delete 
				from $cfgTabAuthContentToCategory  
                where CNTNT_TO_CTGRY_ID_PK = '$contentId' 
				and CNTNT_TO_CTGRY_CNTNTTYP_ID_FK = '$contentTypeId' 
				and CNTNT_TO_CTGRY_CATEGORY_ID_FK = '$categoryId'";
        }
        else
        {
            // delete content
            $sql = "delete from $cfgTabAuthContentToCategory 
                where 
                CNTNT_TO_CTGRY_CATEGORY_ID_FK = $categoryId";
        }
        
        $result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
           $this->setErrorMessage($result->getMessage());
           return false;
        }
        
        return true;
    }
    
    /**
    *	deletes all permissions of this category
    *
    *	@access	public
    *	@param	int $groupId
    *   @return string true/false
    */
    function deletePermissionsFromCategory($categoryId)
    {
        global $cfgTabGroupToCatToPerm;
        
        // check if all values are submitted
        if($categoryId == "")
        {   
            $this->setErrorMessage("no value for categoryId submitted");
            return false;
        }
        
        // check if category exists
        if(!$this->isCategory($categoryId))
        {
            return false;
        } 
        
        // delete permissions
        $sql = "delete 
			from $cfgTabGroupToCatToPerm  
            where CTGRY_TO_PRMSSN_CTGRY_ID_FK = $categoryId";
        //
		$result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
           $this->setErrorMessage($result->getMessage());
           return false;
        }
        
        return true;
    }
        
    function getContentInfo($contentId)
    {
        global $cfgTabAuthContentToCategory;
        
        // check if all values are submitted
        if($contentId == "")
        {   
            $this->setErrorMessage("no value for contentId submitted");
            return false;
        }
        
        $sql = "select * 
			from $cfgTabAuthContentToCategory 
            where CNTNT_TO_CTGRY_ID_PK = $contentId";
		//
        $result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
           $this->setErrorMessage($result->getMessage());
           return false;
        }
        
        while ($row = $result->fetchRow(DB_FETCHMODE_ASSOC)) 
        {
            $contentArray = $row;
        }
        
        return $contentArray;
    }
    
    function getContentFromCategory($contentTypeId, $categoryId)
    {
        global $cfgTabAuthContentToCategory;
        
        // check if all values are submitted
        if($categoryId == "" or $contentTypeId == "")
        {   
            $this->setErrorMessage("no value for categoryId or contentTypeId submitted");
            return false;
        }
        
        // check if category exists
        if(!$this->isCategory($categoryId))
        {
            return false;
        }
        
        // get content
        $sql = "select *  
			from $cfgTabAuthContentToCategory  
            where CNTNT_TO_CTGRY_CATEGORY_ID_FK = '$categoryId' 
			and CNTNT_TO_CTGRY_CNTNTTYP_ID_FK = '$contentTypeId'";
        //
		$result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
           $this->setErrorMessage($sql." ".$result->getMessage());
           return false;
        }
        
        while ($row = $result->fetchRow(DB_FETCHMODE_ASSOC)) 
        {
            $contentArray[] = $row;
        }
        
        return $contentArray;
        
        return true;
    }
    
    /**
    *	returns all content types
    *
    *	@access	public
    *   @return array  
    */
    function getContentTypes()
    {
        global $cfgTabAuthContentTypes;
        
        $sql = "select * 
			from $cfgTabAuthContentTypes";
        //
        $result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
            $this->setErrorMessage($result->getMessage());
            return false;
        }
        
        while ($row = $result->fetchRow(DB_FETCHMODE_ASSOC)) 
        {
            $contentTypeArray[] = $row;
        }
        
        return $contentTypeArray;
    }
    
    /**
    *	returns information about content types
    *
    *	@access	public
    *   @param  int    $contentTypeId 
    *   @return array  
    */
    function getContentTypeInfo($contentTypeId)
    {
        global $cfgTabAuthContentTypes;
        
        $sql = "select * 
			from $cfgTabAuthContentTypes 
            where CONTENTTYPE_ID_PK = '$contentTypeId'";
        //
        $result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
            $this->setErrorMessage($result->getMessage());
            return false;
        }
        
        while ($row = $result->fetchRow(DB_FETCHMODE_ASSOC)) 
        {
            $contentTypeArray = $row;
        }
        
        return $contentTypeArray;
    }
    
    /**
    *	returns information about content types
    *
    *	@access	public
    *   @param  int    $contentTypeId 
    *   @return array  
    */
    function getContent($table)
    {
        $sql = "select * 
			from $table";
        //
        $result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
            $this->setErrorMessage($result->getMessage());
            return false;
        }
        
        while ($row = $result->fetchRow(DB_FETCHMODE_ASSOC)) 
        {
            $contentArray[] = $row;
        }
        
        return $contentArray;
    }
    
    /**
    *	returns permission name
    *
    *	@access	public
    *   @param  int     $id  permission id 
    *   @return string  name of permission
    */
    function getNameForPermissionId($id)
    {
	global $lang_script;
	
        switch($id)
        {
            case 1:
            $permissionName = $lang_script["category.inc.php"]["permissionNameRead"];
            break;
            
            case 2:
            $permissionName = $lang_script["category.inc.php"]["permissionNameWrite"];
            break;
            
            case 4:
            $permissionName = $lang_script["category.inc.php"]["permissionNameExec"];
            break;
        }
        
        return $permissionName;
    }
    
    /**
    *	checks if category exists
    *
    *	@access	public
    *	@param	string	$userid
    *   @return string true/false
    */
    function isCategory($catId)
    {
        global $cfgTabAuthCategory;
        
         // check if user exists
        $sql = "select 1 
			from $cfgTabAuthCategory 
			where CATEGORY_ID_PK = $catId";
		//
        $result = $this->db->query($sql);
        if($result->numRows() != 1)
        {   
            $this->setErrorMessage("category with this id does not exist");
            return false;
        }
        
        return true;
    }
    
    /**
    *	checks if category does not exist
    *
    *	@access	public
    *	@param	string	$groupName
    *	@param	string	$parentId
    *   @return string  true/false
    */
    function existsContent($contentTypeId, $contentId, $categoryId)
    {
        global $cfgTabAuthContentToCategory;
        
        // check if all values are submitted
        if($contentTypeId == "" or $contentId == "" or $categoryId == "")
        {   
            $this->setErrorMessage("no value for contentTypeId or contentId or categoryId submitted");
            return false;
        }
        
        // check if category exists
        $sql = "select 1 
			from $cfgTabAuthContentToCategory 
	        where CNTNT_TO_CTGRY_CATEGORY_ID_FK = '$categoryId' 
			and CNTNT_TO_CTGRY_CNTNTTYP_ID_FK = '$contentTypeId' 
			and CNTNT_TO_CTGRY_CONTENT_ID = '$contentId'";
        //
		$result = $this->db->query($sql);
        if($result->numRows() == 1)
        {   
            $this->setErrorMessage("the content with this id already exists in this category");
            return false;
        }
        
        return true;
    }
    
    /**
    *	checks if category does not exist
    *
    *	@access	public
    *	@param	string	$groupName
    *	@param	string	$parentId
    *   @return string  true/false
    */
    function existsCategory($catName, $parentId)
    {
        global $cfgTabAuthCategory;
        
        // check if all values are submitted
        if($catName == "" or $parentId == "")
        {   
            $this->setErrorMessage("no value for catName or parentId submitted");
            return false;
        }
        
        // check if category exists
        $sql = "select 1 
			from $cfgTabAuthCategory 
	        where CATEGORY_NAME = '$catName' 
			and CATEGORY_PARENT_ID = '$parentId'";
		//
        $result = $this->db->query($sql);
        if($result->numRows() == 1)
        {   
            $this->setErrorMessage("the category with this name and parent already exists");
            return false;
        }
        
        return true;
    }
    
    /**
    *	checks if node has childs
    *
    *	@access	public
     *	@param	int     $id   
    *   @return string  true/false
    */
    function isNode($id)
    {
        global $cfgTabAuthCategory;
        
        $sql = "select 1 
			from $cfgTabAuthCategory 
			where CATEGORY_PARENT_ID = '$id'";
        //
        $result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
            return $this->db->getMessage();
        }
        
        if ($result->numRows()  > 0)
        {
            return true;
        }
                
        return  false;
    }
    
    /**
    *	checks if node has childs and returns array with childs
    *
    *	@access	public
     *	@param	int     $id   group id
    *   @return array   array with child id and name
    */
    function getChilds($id)
    {
        global $cfgTabAuthCategory;
        
        $sql = "select * 
			from $cfgTabAuthCategory 
			where CATEGORY_PARENT_ID = '$id'";
        //
        $result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
            return $this->db->getMessage();
        }
        
        while ($row = $result->fetchRow())
        {
            $childArray[] = $row;
        }
                
        return  $childArray;
    }
    
} // end of class catFunctions
?>
