<?php
// +----------------------------------------------------------------------+
// | sa auth module - group management                                    |
// +----------------------------------------------------------------------+
// | Copyright (c) 2002 serie a - Digitale Medien und Systeme GmbH        |
// +----------------------------------------------------------------------+
// | Authors: Jrn Allert <allert@serie-a.de>                             |
// |          Rene Renk <renk@serie-a.de>                                 |
// +----------------------------------------------------------------------+
// | Description:                                                         |
// +----------------------------------------------------------------------+
//
// $Id: group.inc.php,v 1.1.1.1 2004/07/15 13:35:19 opensurveypilot Exp $

// require all configuration files and libraries
require_once($cfgPathToProjectAdmin."/config/config.inc.php"); // common config file
require_once($cfgPathToConf."/file.inc.php"); // project files
require_once ("DB.php"); // PEAR::DB

/**
* groupFunctions class
*
* User Management Class
*
* @author  Jrn Allert <allert@serie-a.de>
* @author  Rene Renk <renk@serie-a.de>
* @access  public
*/
class groupFunctions
{
    /**
    *	handle for db connection
    *	@var    string  $db
    */
    var $db = "";
    
    /**
    *	error message
    *	@var    string  $errorMessage
    */
    var $errorMessage;
    
    /**
    *	array with ordered group ids
    *	@var    array  $groupTree
    */
    var $groupTree;
    
    /**
    *	constructor 
    *
    *	@access	public
    */
    function groupFunctions()
    {
        $this->errorMessage = "";
        // connect to db
        $this->db = DB::connect(DSN, true);
        if (DB::isError($this->db)) 
        {    
           return false;
        }
    }
    
    /**
    *	gets error message
    *
    *	@access	private
    *	@param	string	$err
    */
    function getErrorMessage($err)
    {
        $this->errorMessage = $err;
    }
    
    /**
    *	adds a group
    *
    *	@access	public
    *	@param	array	$groupArray
    *   @return int $nextId id of the new group
    */
    function addGroup($groupArray)
    {
        global $cfgTabAuthGroup;
        
        // extract submitted values
        $name = $groupArray["name"];
        $parentId = $groupArray["parentId"];
        // check,if all nedded values have been submitted
        if($name == "" or $parentId == "")
        {   
            $this->getErrorMessage("no value for name or parentId submitted");
            return false;
        }
        
        // check if this group already exists
        if(!$this->existsGroup($name, $parentId))
        {   
            return false;
        }
                
        // get next free id
        $nextId = $this->db->nextId($cfgTabAuthGroup);
        if (DB::isError($nextId)) 
        {
            $this->getErrorMessage($result->getMessage());
            return false;
        }
        
        // create insert statement
        $sql = "insert 
			into $cfgTabAuthGroup 
            (GROUP_ID_PK, GROUP_NAME, GROUP_PARENT_ID, GROUP_GROUPTIME) 
            values 
            ('$nextId', '$name', '$parentId', NOW())";
        //
        $result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
           $this->getErrorMessage($result->getMessage());
           return false;
        }
        
        return $nextId;
    }

    /**
    *	updates a group
    *
    *	@access	public
    *	@param	array	$groupArray
    *   @return int $nextId id of the new group
    */
    function updateGroup($groupArray)
    {
        global $cfgTabAuthGroup;
        global $lang_script;
        
        // extract submitted values
        $name = $groupArray["name"];
        $groupId = $groupArray["groupId"];
		
        // check,if all nedded values have been submitted
        if($name == "" or $groupId == "")
        {   
            $this->getErrorMessage("no value for name or groupId submitted");
            return false;
        }
		
        // check if name exists
        $sql = "select 1 
			from $cfgTabAuthGroup  
            where GROUP_NAME = '".$name."'";
		//
        $result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
	        $this->getErrorMessage($result->getMessage());
        	return false;
        }
		if($result->numRows() > 0)
		{
			$this->getErrorMessage($lang_script["group.inc.php"]["nameExistsError"]);
			return false;
		}
		
        // create update statement
        $sql = "update 
			$cfgTabAuthGroup 
            set 
			GROUP_NAME = '".$name."'
            where GROUP_ID_PK = '".$groupId."'";
        //
        $result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
           $this->getErrorMessage($result->getMessage());
           return false;
        }
        
        return true;
    }
	
    /**
    *	deletes a group
    *
    *   use deleteUsersFromGroup($groupId) and deleteCategoriesFromGroup($groupId) 
    *   if you want to delete all content of this group
    *
    *	@access	public
    *	@param	int $groupId
    *   @return string true/false
    */
    function deleteGroup($groupId)
    {
        global $cfgTabAuthGroup;
        
        // check if all values are submitted
        if($groupId == "")
        {   
            $this->getErrorMessage("no value for groupId submitted");
            return false;
        }
        
        // check if group exists
        if(!$this->isGroup($groupId))
        {
            return false;
        } 
        
        // delete group
        $sql = "delete 
			from $cfgTabAuthGroup  
            where GROUP_ID_PK = '$groupId'";
        //
		$result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
           $this->getErrorMessage($result->getMessage());
           return false;
        }
        
        return true;
    }
    
    /**
    *	returns info to this group
    *
    *  returns informationn about group
    *
    *	@access	public
    *	@param	int $groupId
    *   @return array array with group information
    */
    function getGroupInfo($groupId)
    {
        global $cfgTabAuthGroup;
        
        // check if all values are submitted
        if($groupId == "")
        {   
            $this->getErrorMessage("no value for groupId submitted");
            return false;
        }
        
        // check if group exists
        if(!$this->isGroup($groupId))
        {
            return false;
        } 
        
        // get group info
        $sql = "select * 
			from $cfgTabAuthGroup  
            where GROUP_ID_PK = $groupId";
		//
        $result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
           $this->getErrorMessage($result->getMessage());
           return false;
        }
        
        // get num rows
        $groupArray["userNum"] = $this->getUserNum($groupId);
		
        // get table info
        while ($row = $result->fetchRow(DB_FETCHMODE_ASSOC)) 
        {
            foreach($row as $key => $value)
            {
                $groupArray[$key] = $value;
            }
        }
        // add num Groups
		$groupArray["groupNum"] = $this->getGroupNum($groupId);
		
        return $groupArray;
    }
	
    /**
    *	returns number of groups in this group
    *
    *
    *	@access	public
    *   $param  string  groupId
    *   @return int     number of groups
    */
    function getGroupNum($groupId)
    {
        global $cfgTabAuthGroup;
		
        if ($groupId == "")
		{
        $sql = "select * from $cfgTabAuthGroup  
            where 
			GROUP_PARENT_ID = '0'";
		}
		else
		{
        $sql = "select * from $cfgTabAuthGroup  
            where 
			GROUP_PARENT_ID = $groupId";
		}
			
        $result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
            $this->getErrorMessage($result->getMessage());
            return false;
        }
        
        return $result->numRows();
    }
	
    /**
    *	returns number of users in this group
    *
    *
    *	@access	public
    *   $param  string  groupId
    *   @return int     number of users
    */
    function getUserNum($groupId)
    {
        global $cfgTabAuthUser2Group;
         
        $sql = "select 1 
			from $cfgTabAuthUser2Group  
            where USER_TO_GROUP_GROUP_ID_FK = $groupId";
		//
		$result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
            $this->getErrorMessage($result->getMessage());
            return false;
        }
        
        return $result->numRows();
    }
    
    /**
    *	returns ordered groups
    *
    *   recursive function wich orders group from parent to child
    *
    *	@access	public
    *   @return array    ordered array with group id
    */
    function getOrderedGroups($parentId, $startTime)
    {
        global $cfgTabAuthGroup;
	global $lang_script;
         
        $sql = "select * 
			from $cfgTabAuthGroup  
            where GROUP_PARENT_ID = $parentId";
		//
        $result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
            $this->getErrorMessage($sql);
            //$this->getErrorMessage($result->getMessage());
            return false;
        }
        
        $this->groupTree[0] = array("all", $lang_script["group.inc.php"]["allUser"]);
        
        while ($row = $result->fetchRow()) 
        {
            $this->groupTree[] = array($row[0],$row[1]);
            if (time() - $startTime <  10)
            {
                if($this->isNode($row[0]))
                {
                    $this->getOrderedGroups($row[0] , $startTime);
                }
            }
        }
    }

	/**
    *	returns groups in groups data
    *
    *   recursive function wich orders group from parent to child
    *
    *	@access	public
    *   @return array    ordered array with group id
    */
    function getGroups($parentId, $actualNum = 0, $maxRow, $orderby, $direction)
    {
        global $cfgTabAuthGroup;
		
        if($parentId == "")
		{
        $sql = "select * from $cfgTabAuthGroup  
            where GROUP_PARENT_ID = '0'
            order by $orderby $direction 
            limit $actualNum, $maxRow";
		}
		else
		{
        $sql = "select * from $cfgTabAuthGroup  
            where GROUP_PARENT_ID = $parentId
            order by $orderby $direction 
            limit $actualNum, $maxRow";
		}
        $result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
            $this->getErrorMessage($sql);
            //$this->getErrorMessage($result->getMessage());
            return false;
        }
        while ($row = $result->fetchRow(DB_FETCHMODE_ASSOC))
        {
            $groups[] = $row;
        }
        
        return $groups;
    }
	
    /**
    *	checks if node has childs
    *
    *	@access	public
     *	@param	int     $id   
    *   @return string  true/false
    */
    function isNode($id)
    {
        global $cfgTabAuthGroup;
        
        $sql = "select 1 
			from $cfgTabAuthGroup 
			where GROUP_PARENT_ID = '$id'";
        //
        $result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
            return "Fehler. ".$this->db->getMessage();
        }
        
        if ($result->numRows()  > 0)
        {
            return true;
        }
                
        return  false;
    }
    
    /**
    *	checks if node has childs and returns array with childs
    *
    *	@access	public
     *	@param	int     $id   group id
    *   @return array   array with child id and name
    */
    function getChilds($id)
    {
        global $cfgTabAuthGroup;
        
        $sql = "select * 
			from $cfgTabAuthGroup 
			where GROUP_PARENT_ID = '$id'";
        //
        $result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
            return "Fehler. ".$this->db->getMessage();
        }
        
        while ($row = $result->fetchRow())
        {
            $childArray[] = $row;
        }
                
        return  $childArray;
    }
    
    /**
    *	deletes all members of this group
    *
    *	@access	public
    *	@param	int $groupId
    *   @return string true/false
    */
    function deleteUsersFromGroup($groupId)
    {
        global $cfgTabAuthUser2Group;
        
        // check if all values are submitted
        if($groupId == "")
        {   
            $this->getErrorMessage("no value for groupId submitted");
            return false;
        }
        
        // check if group exists
        if(!$this->isGroup($groupId))
        {
            return false;
        } 
        
        // delete group members
        $sql = "delete 
			from $cfgTabAuthUser2Group  
            where USER_TO_GROUP_GROUP_ID_FK = $groupId";
        //
		$result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
           $this->getErrorMessage($result->getMessage());
           return false;
        }
        
        return true;
    }
    
    /**
    *	deletes all categories of this group
    *
    *	@access	public
    *	@param	int $groupId
    *   @return string true/false
    */
    function deleteCategoriesFromGroup($groupId)
    {
        global $cfgTabGroupToCatToPerm;
        
        // check if all values are submitted
        if($groupId == "")
        {   
            $this->getErrorMessage("no value for groupId submitted");
            return false;
        }
        
        // check if group exists
        if(!$this->isGroup($groupId))
        {
            return false;
        } 
        
        // delete group members
        $sql = "delete 
			from $cfgTabGroupToCatToPerm 
            where CTGRY_TO_PRMSSN_GROUP_ID_FK = $groupId";
        //
		$result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
           $this->getErrorMessage($result->getMessage());
           return false;
        }
        
        return true;
    }
    
    /**
    *	adds a user to a group
    *
    *	@access	public
    *	@param	string	$userId     user id
    *	@param	string	$groupId    group id
    *   @return string  true/false
    */
    function addUserToGroup($userId, $groupId)
    {
        global $cfgTabAuthUser2Group;
                
        // check if all values are submitted
        if($userId == "" or $groupId == "")
        {   
            $this->getErrorMessage("no value for userId or groupId submitted");
            return false;
        }
        
        // check if user exists, group exists and if user is not already in this group
        if(!$this->isUser($userId) or !$this->isGroup($groupId) or !$this->isUSerInGroup($userId, $groupId))
        {
            $this->getErrorMessage("user or group does not exist. or user is already in group");
            return false;
        } 
        
        // get next free id
        $nextId = $this->db->nextId($cfgTabAuthUser2Group);
        if (DB::isError($nextId)) 
        {
            $this->getErrorMessage($result->getMessage());
            return false;
        }
                
        // insert user and group in table user2group
        $sql = "insert 
			into $cfgTabAuthUser2Group 
            (USER_TO_GROUP_ID_PK, USER_TO_GROUP_USER_ID_FK, USER_TO_GROUP_GROUP_ID_FK) 
            values 
            ('$nextId', '$userId', '$groupId')";
        //
		$result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
           $this->getErrorMessage($result->getMessage());
           return false;
        }
        
        return true;
    }
    
    /**
    *	deletes a user from a group
    *
    *	@access	public
    *	@param	string	$userId
    *	@param	string	$groupId
    *   @return string true/false
    */
    function deleteUserFromGroup($userId, $groupId)
    {
        global $cfgTabAuthGroup;
        
        // check if all values are submitted
        if($userId == "" or $groupId == "")
        {   
            $this->getErrorMessage("no value for userId or grouptId submitted");
            return false;
        }
        
        // check if user exists, group exists and if user is in this group
        if(!$this->isUser($userId) or !$this->isIdGroup($groupId) or $this->isUSerInGroup($userId, $groupId))
        {
            return false;
        }
        
        // insert user and group in table user2group
        $sql = "delete 
			from $cfgTabAuthUser2Group 
            where USER_TO_GROUP_USER_ID_FK = '$userId' 
			and USER_TO_GROUP_GROUP_ID_FK = '$groupId'";
        //
		$result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
           $this->getErrorMessage($result->getMessage());
           return false;
        }
        
        return true;
    }
    
    /**
    *	get user ids from a group
    *
    *	@access	public
    *	@param	string	$groupId
    *   @return array $userInGroup  Array with all user ids and names
    */
    function getUsersInGroup($groupId)
    {
        global $cfgTabAuthUser, $cfgTabAuthUser2Group;
        
        // check if all values are submitted
        if($groupId == "")
        {   
            $this->getErrorMessage("no value for groupId submitted");
            return false;
        }
        
        // check if group exists
        if(!$this->isGroup($groupId))
        {
            return false;
        }
        
        // fetch all users in this group
        $sql = "select $cfgTabAuthUser.USER_ID_PK, 
            $cfgTabAuthUser.USER_FIRSTNAME, 
            $cfgTabAuthUser.USER_LASTNAME, 
			$cfgTabAuthUser.USER_LOGIN 
            from $cfgTabAuthUser, $cfgTabAuthUser2Group 
            where 
            $cfgTabAuthUser.USER_ID_PK = $cfgTabAuthUser2Group.USER_TO_GROUP_USER_ID_FK and 
            $cfgTabAuthUser2Group.USER_TO_GROUP_GROUP_ID_FK = $groupId
            ";
  		//
        $result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
           $this->getErrorMessage($result->getMessage());
           return false;
        }
        while ($row = $result->fetchRow())
        {
            $userInGroup[] = $row;
        }
        
        return $userInGroup;
    }
    
    /**
    *	adds the group to a category
    *
    *	@access	public
    *	@param	string	$groupId
    *	@param	string	$catId
    *   @param  string  $permId default to 1 (read)
    *   @return string true/false
    */
    function addGroupToCat($groupId, $catId, $permId = 1)
    {
        global $cfgTabGroupToCatToPerm;
        
        // check if all values are submitted
        if($groupId == "" or $catId == "" or $permId == "")
        {   
            $this->getErrorMessage("no value for grouptId or $catId submitted");
            return false;
        }
        
        // check if group and cat exists
        if(!$this->isGroup($groupId) or !$this->isCat($catId))
        {
            return false;
        }
        
        // get next free id
        $nextId = $this->db->nextId($cfgTabGroupToCatToPerm);
        if (DB::isError($nextId)) 
        {
            $this->getErrorMessage($result->getMessage());
            return false;
        }
        
        // add group to category
        $sql = "insert 
			into $cfgTabGroupToCatToPerm 
            (CTGRY_TO_PRMSSN_ID_PK, CTGRY_TO_PRMSSN_GROUP_ID_FK, CTGRY_TO_PRMSSN_CTGRY_ID_FK, CTGRY_TO_PRMSSN_PERMSSN_ID_FK) 
            values 
            ('$nextId', '$groupId', '$catId', '$permId'";
		//
        $result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
           $this->getErrorMessage($result->getMessage());
           return false;
        }
        
        return $nextId;
    }
    
    /**
    *	deletes a group from a category
    *
    *	@access	public
    *	@param	string	$groupId
    *	@param	string	$catId
    *   @return string true/false
    */
    function deleteGroupfromCat($groupId, $catId)
    {
        global $cfgTabGroupToCatToPerm;
        
        // check if all values are submitted
        if($groupId == "" or $catId == "")
        {   
            $this->getErrorMessage("no value for groupId or $catId submitted");
            return false;
        }
        
        // check if group and cat exists
        if(!$this->isGroup($groupId) or !$this->isCat($catId))
        {
            return false;
        }
        
        // delete group from category
        $sql = "delete 
			from $cfgTabGroupToCatToPerm 
            where CTGRY_TO_PRMSSN_GROUP_ID_FK = '$groupId' 
			and CTGRY_TO_PRMSSN_CTGRY_ID_FK = '$catId'";
        //
		$result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
           $this->getErrorMessage($result->getMessage());
           return false;
        }
        
        return $nextId;
    }
    
    /**
    *	checks if user is in user table
    *
    *	@access	public
    *	@param	string	$userid
    *   @return string true/false
    */
    function isUSer($userId)
    {
        global $cfgTabAuthUser;
        
        // check if all values are submitted
        if($userId == "")
        {   
            $this->getErrorMessage("no value for userId submitted");
            return false;
        }
        
        // check if user exists
        $sql = "select 1 
			from $cfgTabAuthUser 
			where USER_ID_PK = $userId";
		//
        $result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
            $this->getErrorMessage($result->getMessage());
            return false;
        }
        if($result->numRows() != 1)
        {   
            $this->getErrorMessage("user with this id does not exist");
            return false;
        }
        
        return true;
    }
    
    /**
    *	checks if group is in group table
    *
    *	@access	public
    *	@param	string	$groupId
    *   @return string true/false
    */
    function isGroup($groupId)
    {
        global $cfgTabAuthGroup;
        
        // check if all values are submitted
        if($groupId == "")
        {   
            $this->getErrorMessage("no value for groupId submitted");
            return false;
        }
        
        // check if group exists
        $sql = "select 1 
			from $cfgTabAuthGroup 
			where GROUP_ID_PK = $groupId";
		//
        $result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
            $this->getErrorMessage($result->getMessage());
            return false;
        }
        if($result->numRows() != 1)
        {   
            $this->getErrorMessage("group with this id does not exist");
            return false;
        }
        
        return true;
    }
    
    /**
    *	checks if group exists
    *
    *	@access	public
    *	@param	string	$groupName
    *	@param	string	$parentId
    *   @return string true/false
    */
    function existsGroup($groupName, $parentId)
    {
        global $cfgTabAuthGroup;
        
        // check if all values are submitted
        if($groupName == "" or $parentId == "")
        {   
            $this->getErrorMessage("$groupName, $parentId no value for groupName or parentId submitted");
            return false;
        }
        // check if group exists
        $sql = "select 1 from $cfgTabAuthGroup 
	        where GROUP_NAME = '$groupName' 
			and GROUP_PARENT_ID = '$parentId'";
        //
		$result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
            $this->getErrorMessage($result->getMessage());
            return false;
        }
        if($result->numRows() == 1)
        {   
            $this->getErrorMessage("the group with this name and parent already exists");
            return false;
        }
        
        return true;
    }
    
    
    /**
    *	checks if group is in group table
    *
    *	@access	public
    *	@param	string	$groupId
    *   @return string true/false
    */
    function isCat($catId)
    {
        global $cfgTabAuthCategory;
        
        // check if cat exists
        $sql = "select 1 
			from $cfgTabAuthCategory 
			where CATEGORY_ID_PK = $catId";
		//
        $result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
            $this->getErrorMessage($result->getMessage());
            return false;
        }
        if($result->numRows() != 1)
        {   
            $this->getErrorMessage("a category with this id does not exist");
            return false;
        }
        
        return true;
    }
    
    /**
    *	checks if user is in grouzp
    *
    *	@access	public
    *	@param	string	$userid
    *   @return string true/false
    */
    function isUSerInGroup($userId, $groupId)
    {
        global $cfgTabAuthUser2Group;
        
        // check if user already in group
        $sql = "select 1 
			from $cfgTabAuthUser2Group 
			where USER_TO_GROUP_USER_ID_FK = $userId 
			and USER_TO_GROUP_GROUP_ID_FK = $groupId";
        //
		$result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
            $this->getErrorMessage($result->getMessage());
            return false;
        }
        if($result->numRows() == 1)
        {   
            $this->getErrorMessage("user exists already in this group");
            return false;
        }
        
        return true;
    }
    
} // end of class groupFunctions
?>
