<?php
// +----------------------------------------------------------------------+
// | sa auth module - permission management                               |
// +----------------------------------------------------------------------+
// | Copyright (c) 2002 serie a - Digitale Medien und Systeme GmbH        |
// +----------------------------------------------------------------------+
// | Authors: Jrn Allert <allert@serie-a.de>                             |
// |          Rene Renk <renk@serie-a.de>                                 |
// +----------------------------------------------------------------------+
// | Description:                                                         |
// +----------------------------------------------------------------------+
//
// $Id: permission.inc.php,v 1.1.1.1 2004/07/15 13:35:19 opensurveypilot Exp $

// require all configuration files and libraries
require_once($cfgPathToProjectAdmin."/config/config.inc.php"); // common config file
require_once($cfgPathToConf."/file.inc.php"); // project files
require_once ("DB.php"); // PEAR::DB

/**
* permFunctions class
*
* Permission Management Class
*
* @author  Jrn Allert <allert@serie-a.de>
* @author  Rene Renk <renk@serie-a.de>
* @access  public
*/
class permissionFunctions
{
    /**
    *	error message
    *	@var    string  $errorMessage
    */
    var $errorMessage;
    
    /**
    *	constructor 
    *
    *	@access	public
    */
    function permissionFunctions()
    {
        $this->errorMessage = "";
        // connect to db
        $this->db = DB::connect(DSN, true);
        if (DB::isError($this->db)) 
        {    
           return false;
        }
    }
    
    /**
    *	gets error message
    *
    *	@access	private
    *	@param	string	$err
    */
    function setErrorMessage($err)
    {
        $this->errorMessage = $err;
    }
    
    /**
    *	adds a permission
    *
    *	@access	public
    *	@param	int     $groupId
    *	@param	int     $categoryId
    *   @param  int     $permissionId
    *   @return int     $nextId id of the new permission
    */
    function addPermission($groupId, $categoryId, $permissionId)
    {
        global $cfgTabGroupToCatToPerm;
        
        // check,if all nedded values has been submitted
        if($groupId == "" or $permissionId == "" or $categoryId == "")
        {   
            $this->setErrorMessage("no value for groupId or permissionId or $categoryId submitted");
            return false;
        }
        
        // permsission Id must be 1,2 or 4
        if($permissionId != "1" and $permissionId != "2" and $permissionId != "4")
        {   
            $this->setErrorMessage("permsission Id must be 1,2 or 4");
            return false;
        }
        
        // check if this permission already exists
        if(!$this->existsPermission($groupId, $categoryId, $permissionId))
        {   
            return false;
        }
        
        // check if group exists
        if(!$this->isGroup($groupId))
        {   
            return false;
        }
                
        // get next free id
        $nextId = $this->db->nextId($cfgTabGroupToCatToPerm);
        if (DB::isError($nextId)) 
        {
            $this->setErrorMessage($result->getMessage());
            return false;
        }
        
        // create insert statement
        $sql = "insert 
			into $cfgTabGroupToCatToPerm 
            (CTGRY_TO_PRMSSN_ID_PK, CTGRY_TO_PRMSSN_GROUP_ID_FK, CTGRY_TO_PRMSSN_CTGRY_ID_FK, CTGRY_TO_PRMSSN_PERMSSN_ID_FK, CTGRY_TO_PRMSSN_PERMSSNTIME) 
            values 
            ('$nextId', '$groupId', '$categoryId', '$permissionId', NOW())";
        //
        $result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
           $this->setErrorMessage($result->getMessage());
           return false;
        }
        
        return $nextId;
    }
    
    /**
    *	deletes a permission
    *
    *	@access	public
    *	@param	int $permissionId
    *   @return string true/false
    */
    function deletePermission($permissionId)
    {
        global $cfgTabGroupToCatToPerm;
        
        // check if all values are submitted
        if($permissionId == "")
        {   
            $this->setErrorMessage("no value for $permissionId submitted");
            return false;
        }
        
        // check if categiory exists
        if(!$this->isPermission($permissionId))
        {
            return false;
        } 
        
        // delete permission
        $sql = "delete 
			from $cfgTabGroupToCatToPerm  
            where CTGRY_TO_PRMSSN_ID_PK = $permissionId";
		//
        $result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
           $this->setErrorMessage($result->getMessage());
           return false;
        }
        
        return true;
    }
    
    /**
    *	edit permission 
    *
    *	@access	public
    *	@param	string	$permissionId
    *	@param	string	$groupId
    *	@param	string	$permissionModeId
    *   @return string true/false
    */
    function editPermission($permissionId, $groupId, $permissionModeId)
    {
        global $cfgTabGroupToCatToPerm;
        
       // check if all values are submitted
        if($permissionId == "" or $groupId == "" or $permissionModeId == "")
        {   
            $this->setErrorMessage("no value for $permissionId or $groupId or $permissionModeId submitted");
            return false;
        }
        
        // check if $permId is a valid int
        if($permissionModeId != 1 and $permissionModeId != 2 and $permissionModeId != 4)
        {   
            $this->setErrorMessage("permissionId must be 1, 2 or 4.");
            return false;
        }
        
        // check if $permissionId exists
        if(!$this->isPermission($permissionId))
        {
            return false;
        }
        
        // update permission
        $sql = "update 
			$cfgTabGroupToCatToPerm 
            set 
            CTGRY_TO_PRMSSN_GROUP_ID_FK = '$groupId', 
            CTGRY_TO_PRMSSN_PERMSSN_ID_FK = '$permissionModeId', 
			CTGRY_TO_PRMSSN_PERMSSNTIME = NOW() 
            where CTGRY_TO_PRMSSN_ID_PK = '$permissionId'";
        //  
        $result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
           $this->setErrorMessage($result->getMessage());
           return false;
        }
        
        return true;
    }
    
    /**
    *	returns info to this permission
    *
    *	@access	public
    *	@param	int $permissionId
    *   @return array array with permission information
    */
    function getPermissionInfo($permissionId)
    {
       global $cfgTabGroupToCatToPerm;
       global $cfgTabAuthGroup;
        
        // check if all values are submitted
        if($permissionId == "")
        {   
            $this->setErrorMessage("no value for permissionId submitted");
            return false;
        }
        
        // check if permission exists
        if(!$this->isPermission($permissionId))
        {
            return false;
        } 
        
        // get permission info
        $sql = "select $cfgTabGroupToCatToPerm.*, $cfgTabAuthGroup.GROUP_NAME 
            from $cfgTabGroupToCatToPerm, $cfgTabAuthGroup 
            where $cfgTabGroupToCatToPerm.CTGRY_TO_PRMSSN_ID_PK = $permissionId 
			and $cfgTabGroupToCatToPerm.CTGRY_TO_PRMSSN_GROUP_ID_FK = $cfgTabAuthGroup.GROUP_ID_PK";
        //
        $result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
           $this->setErrorMessage($result->getMessage());
           return false;
        }
                
        // get table info
        while ($row = $result->fetchRow(DB_FETCHMODE_ASSOC)) 
        {
            $permissionArray = $row;
        }
        
        return $permissionArray;
    }
    
    /**
    *	returns permission mode
    *
    *   return values are:
    *   0:  no permission
    *   1:  read
    *   2:  write
    *   4:  execute (delete and everything else)
    *
    *	@access	public
    *	@param	string	$userId
    *	@param	string	$contentId
    *   @return int     permission mode
    */
    function getPermissionMode($userId, $contentId, $contentTypeId)
    {
        global $cfgTabGroupToCatToPerm;
        global $cfgTabAuthContentToCategory;
        global $cfgTabAuthGroup;
        global $cfgTabAuthUser2Group;
                
        // if userstatus is administrator(8) return 4
        if($_SESSION["sessionUserStatusId"] == "8")
        {
            return 4;
        }
        
        // check if all values are submitted
        if($userId == "" or $contentId == "" or $contentTypeId == "")
        {   
            $this->setErrorMessage("no value for userId, contentId, contentTypeId submitted");
            return false;
        }
        
        $sql = "select 
            max($cfgTabGroupToCatToPerm.CTGRY_TO_PRMSSN_PERMSSN_ID_FK) as greatest 
            from $cfgTabGroupToCatToPerm, $cfgTabAuthContentToCategory, $cfgTabAuthGroup, $cfgTabAuthUser2Group 
            where $cfgTabGroupToCatToPerm.CTGRY_TO_PRMSSN_CTGRY_ID_FK = $cfgTabAuthContentToCategory.CNTNT_TO_CTGRY_CATEGORY_ID_FK 
			and $cfgTabAuthGroup.GROUP_ID_PK = $cfgTabGroupToCatToPerm.CTGRY_TO_PRMSSN_GROUP_ID_FK 
			and $cfgTabAuthGroup.GROUP_ID_PK = $cfgTabAuthUser2Group.USER_TO_GROUP_GROUP_ID_FK 
			and $cfgTabAuthUser2Group.USER_TO_GROUP_USER_ID_FK = '$userId' 
			and $cfgTabAuthContentToCategory.CNTNT_TO_CTGRY_CNTNTTYP_ID_FK = '$contentTypeId' 
			and $cfgTabAuthContentToCategory.CNTNT_TO_CTGRY_CONTENT_ID = '$contentId'
            ";
        //   
        $result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
            $this->setErrorMessage($sql." ".$result->getMessage());
            return false;
        }
        
        while ($row = $result->fetchRow(DB_FETCHMODE_ASSOC)) 
        {
            $permissionMode = $row["greatest"];
        }
        
        return $permissionMode;
    }
    
    /**
    *	returns permission name
    *
    *	@access	public
    *   @param  int     $id  permission id 
    *   @return string  name of permission
    */
    function getNameForPermissionId($id)
    {
	global $lang_script;
	
        switch($id)
        {
            case 1:
            $permissionName = $lang_script["permission.inc.php"]["permissionNameRead"];
            break;
            
            case 2:
            $permissionName = $lang_script["permission.inc.php"]["permissionNameWrite"];
            break;
            
            case 4:
            $permissionName = $lang_script["permission.inc.php"]["permissionNameExec"];
            break;
        }
        
        return $permissionName;
    }
    
    /**
    *	checks if permission exists
    *
    *	@access	public
    *	@param	string	$groupName
    *	@param	string	$parentId
    *   @return string true/false
    */
    function existsPermission($groupId, $categoryId, $permissionId)
    {
        global $cfgTabGroupToCatToPerm;
        
        // check if all values are submitted
        if($groupId == "" or $categoryId == "" or $permissionId == "")
        {   
            $this->setErrorMessage("no value for groupId or permissionId or categoryId submitted");
            return false;
        }
        
        // check if permission exists
        $sql = "select 1 
			from $cfgTabGroupToCatToPerm 
	        where CTGRY_TO_PRMSSN_GROUP_ID_FK = '$groupId' 
			and CTGRY_TO_PRMSSN_CTGRY_ID_FK = '$categoryId' 
			and CTGRY_TO_PRMSSN_PERMSSN_ID_FK = '$permissionId'";
		//
        $result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
            $this->setErrorMessage($result->getMessage());
            return false;
        }
        if($result->numRows() > 0)
        {   
            $this->setErrorMessage("the group with this name and parent already exists");
            return false;
        }
        
        return true;
    }
    
    /**
    *	checks if permission is in permission table
    *
    *	@access	public
    *	@param	string	$permissionId
    *   @return string true/false
    */
    function isPermission($permissionId)
    {
        global $cfgTabGroupToCatToPerm;
        
        // check if all values are submitted
        if($permissionId == "")
        {   
            $this->setErrorMessage("no value for permissionId submitted");
            return false;
        }
        if (DB::isError($result)) 
        {
            $this->setErrorMessage($result->getMessage());
            return false;
        }
        // check if permissionexists
        $sql = "select 1 
			from $cfgTabGroupToCatToPerm 
			where CTGRY_TO_PRMSSN_ID_PK = $permissionId";
		//
        $result = $this->db->query($sql);
        
        if($result->numRows() != 1)
        {   
            $this->setErrorMessage("permission with this id does not exist");
            return false;
        }
        
        return true;
    }
    
    /**
    *	checks if group is in group table
    *
    *	@access	public
    *	@param	string	$groupId
    *   @return string true/false
    */
    function isGroup($groupId)
    {
        global $cfgTabAuthGroup;
        
        // check if all values are submitted
        if($groupId == "")
        {   
            $this->setErrorMessage("no value for groupId submitted");
            return false;
        }
        
        // check if group exists
        $sql = "select 1 
			from $cfgTabAuthGroup 
			where GROUP_ID_PK = $groupId";
        //
		$result = $this->db->query($sql);
        if($result->numRows() != 1)
        {   
            $this->setErrorMessage("group with this id does not exist");
            return false;
        }
        
        return true;
    }
    
} // end of class permFunctions
