<?php
// +----------------------------------------------------------------------+
// | sa auth module - user management                                     |
// +----------------------------------------------------------------------+
// | Copyright (c) 2002 serie a - Digitale Medien und Systeme GmbH        |
// +----------------------------------------------------------------------+
// | Authors: J?rn Allert <allert@serie-a.de>                             |
// |          Rene Renk <renk@serie-a.de>                                 |
// +----------------------------------------------------------------------+
// | Description: XMLRPC-Client class for user authentification and       |
// |          permission management .                                     |
// +----------------------------------------------------------------------+
//
// $Id: user.inc.php,v 1.1.1.1 2004/07/15 13:35:20 opensurveypilot Exp $

require_once($cfgPathToProjectAdmin."/config/config.inc.php"); // common config file
require_once($cfgPathToConf."/file.inc.php"); // project files
require_once($cfgPathToAuthFiles."/config/auth.file.inc.php"); // auth files
require_once($cfgPathToLib."/xmlrpc/xmlrpc.inc"); // xmlrpc client library

require_once ("DB.php"); // PEAR::DB
require_once("HTML/QuickForm.php"); // PEAR::HTML_QuickForm

define("FILEXMLSERVER",$fileAuthXmlrpcServer);
define("CFGIP",$cfgXmlrpcServer);
define("FILEAUTHLOGINFORM",$cfgUrlToProject."/".$fileAuthLogin);
define("SERVERPATH",$cfgPathToXmlrpcServer);
/**
* userFunctions class
*
* XMLRPC-Client class for user authentification management
*
* @author  J?rn Allert <allert@serie-a.de>
* @author  Rene Renk <renk@serie-a.de>
* @access  public
*/
class userFunctions
{
    /**
    *	handle for db connection
    *	@var    string  $db
    */
    var $db = "";
    
    /**
    *	handle for xmlrpc client
    *	@var    string  $xmlrpcClient
    */
    var $xmlrpcClient = "";
    
    /**
    *	switch xmlrp debug on or of 1/0
    *	@var	string	$xmlrpcDebug
    */
    var $xmlrpcDebug = 0;
    
    /**
    *	sended xmlrpc message
    *	@var	string	$xmlrpcMessage
    */
    var $xmlrpcMessage = "";
    
    /**
    *	received xmlrpc response
    *	@var	string	$xmlrpcResponse
    */
    var $xmlrpcResponse= "";
    
    /**
    *	value of xmlrpc response
    *	@var	string	$xmlrpcValue
    */
    var $xmlrpcValue = "";
    
    /**
    *	create new userFunctions object 
    *
    *	@access	public
    */
    function userFunctions()
    {
        // create xmlrpc client
        $this->xmlrpcClient = $this->createXmlrpcClient();
        // set xmlrpc debug mode
        $this->xmlrpcClient->setDebug($this->xmlrpcDebug);
        // connect to db
        $this->db = DB::connect(DSN, true);
        if (DB::isError($this->db)) 
        {   
           echo "dbFehler"; 
           return false;
        }
    }
    
    /**
    *	formats the user Array
    *   puts all user data in a xmlrpc array.
    *   You have to change $arayFieldNames, if there are changes on the db
    *
    *	@access	private
    *	@param	array	$userArray
    *	@return	array	puts user Arry in xmlrpc values
    */
    function formatUserArray($userArray)
    {
        $arrayFieldNames = array("userLogin", 
            "userPassword", "USER_ID_PK", "USER_LOGIN", "USER_PASSWORD", 
            "USER_USERSTATUS_ID_FK", "USER_LANGUAGE_ID_FK", "USER_FIRSTNAME", "USER_LASTNAME", 
            "USER_COMPANY", "USER_POSITION", "USER_STREET", "USER_POSTALCODE", 
            "USER_CITY", "USER_EMAIL", "USER_PHONE", "USER_FAX", 
            "USER_MOBILE", "USER_HOMEPAGE");
        
        while(list($key, $val) = each($arrayFieldNames))
        {
            if(isset($userArray[$val]))
            {
                $formattedArray[$key] = new sa_xmlrpcval($userArray[$val], "string");
            }
            else
            {
                $formattedArray[$key] = new sa_xmlrpcval("{not_set}", "string");
            }
        }
        
        return $formattedArray;
    }
    
    /**
    *	encode userPassword
    *   uses mhash SHA1
    *
    *	@access	public
    *	@param	string	$login	users password
    *	@return	string	encoded user password
    */
    function encodePassword($password) 
    {
        $hash = base64_encode(mhash(MHASH_SHA1, $password));
        return $hash;
    }
        
    /**
    *	fetch user data
    *   calls function user.fetchUser on the xmlrpc server
    *
    *	@access	public
    *	@param	string	$login	users login name
    *	@param	string	$password	users password
    *	@return	array	returns userdata via xmlrpc
    */
    function fetchUser($login, $password, $fetchId)
    {
        $options = array( 
            new sa_xmlrpcval($login, "string"),
            new sa_xmlrpcval($password, "string"), 
            new sa_xmlrpcval($fetchId, "string")
            );
        $this->xmlrpcMessage = $this->createXmlrpcMessage($options, 'user.fetchUser');
    }
    
    /**
    *	update user data
    *   calls function user.updateUser on the xmlrpc server
    *
    *	@access	public
    *	@param	array	$userArray  array with all user datas to be changed
    *	@return	array	needed vars for authentication (via xmlrpc)
    */
    function updateUser($userArray)
    {
        $formattedArray = $this->formatUserArray($userArray);
        $this->xmlrpcMessage = $this->createXmlrpcMessage($formattedArray, 'user.updateUser');
    }
    
    /**
    *	writes new user
    *   calls function user.writeUser on the xmlrpc server
    *
    *	@access	public
    *	@param	array	$userArray  array with all user datas for new entry
    *	@return	int 	new user id (via xmlrpc)
    */
    function addUser($userArray)
    {
        $formattedArray = $this->formatUserArray($userArray);
        $this->xmlrpcMessage = $this->createXmlrpcMessage($formattedArray, 'user.addUser');
    }
    
    /**
    *	delete user
    *   calls function user.deleteUser on the xmlrpc server
    *
    *	@access	public
    *	@param	array	$userArray  array with all user datas for new entry
    *	@return	int 	new user id (via xmlrpc)
    */
    function deleteUser($login, $password, $userId)
    {
        $options = array( 
            new sa_xmlrpcval($login, "string"),
            new sa_xmlrpcval($password, "string"), 
            new sa_xmlrpcval($userId, "string")
            );
        $this->xmlrpcMessage = $this->createXmlrpcMessage($options, 'user.deleteUser');
    }
    
    function deleteUsersfromGroup($userId)
    {
        global $cfgTabAuthUser2Group ;
        
        // check if all values are submitted
        if($userId == "")
        {   
            $this->getErrorMessage("no value for userId submitted");
            return false;
        }
        
        // check if user exists
        if(!$this->isUser($userId))
        {
            return false;
        } 
        
        // delete users from group
        $sql = "delete 
			from $cfgTabAuthUser2Group   
            where USER_TO_GROUP_USER_ID_FK = '$userId'";
        //
		$result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
           $this->getErrorMessage($result->getMessage());
           return false;
        }
        
        return true;
    }
    
    /**
    *	checks if user is existse
    *
    *	@access	public
    *	@param	string	$userLogin
     *	@param	string	$userPassword  
    *   @return string true/false
    */
    function isUser($userLogin, $userPassword)
    {
        global $cfgTabAuthUser;
        
        // check if all values are submitted
        if($userLogin== "" or $userPassword == "")
        {   
            return false;
        }
        
        // check if user exists
        $sql = "select USER_ID_PK 
			from $cfgTabAuthUser 
			where USER_LOGIN = '$userLogin' 
			and USER_PASSWORD = '$userPassword'";
		//
        $result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
            return $result->getMessage();
        }
        if($result->numRows() != 1)
        {   
            return false;
        }
        while ($row = $result->fetchRow(DB_FETCHMODE_ASSOC)) 
        {
            $id = $row["USER_ID_PK"];
        }
        
        return $id;
    }
    
    /**
    *	checks if user has a valid session id
    *
    *	@access	public
    *	@param	string	$userSession  session id
    *	@param	string	$sessionId	session id saved in session
    */
    function getAuthentication($userSession, $sessionId)
    {
        if ($userSession == $sessionId)
    	{
    		return true;
    	}
    	else
    	{
    		return false;
    	}
    }
    
    /**
    *	get all user data for overview
    *
    *	@access	public
    *	@param	string	$actualNum
    *	@param	string	$maxRow 
    *   @param  string  $orderby
    *	@param	array	array with all userdata
    */
    function getUsersForOverview($groupId, $actualNum = 0, $maxRow, $orderby, $direction)
    {
        global $cfgTabAuthUser;
        global $cfgTabAuthUser2Group;
        
        // check input
        if($groupId == "")
        {
            // fetch all userdata
            $sql = "select * 
                from $cfgTabAuthUser 
                order by $orderby $direction 
                limit $actualNum, $maxRow"; 
        }
        else
        {
            // fetch all userdata
            $sql = "select 
                $cfgTabAuthUser.* 
                from $cfgTabAuthUser, $cfgTabAuthUser2Group 
                where $cfgTabAuthUser.USER_ID_PK = $cfgTabAuthUser2Group.USER_TO_GROUP_USER_ID_FK 
				and $cfgTabAuthUser2Group.USER_TO_GROUP_GROUP_ID_FK = '$groupId' 
                order by $orderby $direction 
                limit $actualNum, $maxRow"; 
        }

        $result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
            return $sql." ".$result->getMessage();
        }
        $k = 0;
        while ($row = $result->fetchRow(DB_FETCHMODE_ASSOC)) 
        {
            $userArray[$k]["USER_ID_PK"] = $row["USER_ID_PK"];
            $userArray[$k]["USER_LASTNAME"] = $row["USER_LASTNAME"];
            $userArray[$k]["USER_FIRSTNAME"] = $row["USER_FIRSTNAME"];
            $userArray[$k]["USER_COMPANY"] = $row["USER_COMPANY"];
            $userArray[$k]["USER_USERSTATUS_ID_FK"] = $row["USER_USERSTATUS_ID_FK"];
            $userArray[$k]["USER_LOGIN"] = $row["USER_LOGIN"];
	    $userArray[$k]["USER_DATE"] = $row["USER_DATE"];
            $k++;
        }
        
        return$userArray;
    }
    
    /**
    *	returns number of users
    *
    *	@access	public
    *   @return string  number of users in table $cfgTabAuthUser
    */
    function getUserNum()
    {
        global $cfgTabAuthUser;
        
        // fetch all users
        $sql = "select * 
			from $cfgTabAuthUser";
		// 
        $result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
            return false;
        }
        $numRows = $result->numRows();
        
        return $numRows;
    }
    
    /**
    *	returns status name
    *
    *	@access	public
    *   @param  int     $id  status id 
    *   @return string  name of status
    */
    function getNameForStatusId($id)
    {
	global $lang_script;
	
        switch($id)
        {
            case 1:
            $statusName = $lang_script["user.inc.php"]["statusNameGuest"];
            break;
            
            case 2:
            $statusName = $lang_script["user.inc.php"]["statusNameUser"];
            break;
            
            case 4:
            $statusName = $lang_script["user.inc.php"]["statusNameProjektManager"];
            break;
            
            case 8:
            $statusName = $lang_script["user.inc.php"]["statusNameAdministrator"];
            break;
        }
        
        return $statusName;
    }
    
    /**
    *	returns language name
    *
    *	@access	public
    *   @param  int     $id  language id 
    *   @return string  name of language
    */
    function getNameForLangId($id)
    {
    	global $lang_script;

        switch($id)
        {
            case 1:
            $languageName = $lang_script["user.inc.php"]["langGerman"];
            break;
            
            case 2:
            $languageName = $lang_script["user.inc.php"]["langEnglish"];
            break;
            
        }
        
        return $languageName;
    }
    
    /**
    *	get info for user language
    *
    *	@access	public
    *   @param  int     $id  language id 
    *   @return array   name of language
    */
    function getUserLanguage($id)
    {
        global $cfgTabAuthLanguage;
        
        $sql = "select * 
            from $cfgTabAuthLanguage 
            where LANGUAGE_ID_PK = $id";
        $result = $this->db->query($sql);
        if (DB::isError($result)) 
        {
            return false;
        }
        while ($row = $result->fetchRow(DB_FETCHMODE_ASSOC))
        {
            $userLanguageArray = $row;    
        }
        
        return $userLanguageArray;
    }
    
    /**
    *	creates the xmlrpc client
    *
    *	@access	private
    */
    function createXmlrpcClient()
    {
        $serverPath = SERVERPATH;
        return new sa_xmlrpc_client($serverPath.FILEXMLSERVER, CFGIP, 80);
        //return new sa_xmlrpc_client($clientPath.FILEXMLSERVER, CFGIP, 443);
    }
    
    /**
    *	creates the xmlrpc message
    *
    *	@access	private
    */
    function createXmlrpcMessage($options, $method)
    {
        $message = new sa_xmlrpcmsg($method, $options); 
        return $message; 
    }
    
    /**
    *	sends the xmlrpc message
    *
    *	@access	public
    */
    function sendXmlrpcMessage()
    {
        $this->xmlrpcResponse = $this->xmlrpcClient->send($this->xmlrpcMessage);
        //with curl and https (set port to 443 while creating the xmlrpc server)
        //$this->xmlrpcResponse = $this->xmlrpcClient->send($this->xmlrpcMessage, 0, 'https'); 
    }
    
    /**
    *	receives the xmlrpc message
    *
    *	@access	public
    *   @return array   xmlrpc response
    */
    function receiveXmlrpcMessage()
    {
        $this->xmlrpcValue = $this->xmlrpcResponse->value();
        if(!$this->xmlrpcResponse->faultCode())
        {
            if ($this->xmlrpcValue->kindOf() == "struct")
            {
                $this->xmlrpcValue->structreset();
                while(list($key,$value)=$this->xmlrpcValue->structeach())
                {
                    if ($value->kindOf() == "scalar")
                    {
                        $responseArray[$key] = $value->scalarval();
                    }
                }
            }
            else
            {
                $v = $this->xmlrpcResponse->value();
                $responseArray[0] = 1;
                $responseArray[1] = $v->scalarval();
            }
        } 
        else 
        {
            // no valid user, return errorcode and message
            $responseArray[0] = 0;
            $responseArray[1] = $this->xmlrpcResponse->faultCode();
            $responseArray[2] = $this->xmlrpcResponse->faultString();
        }
        return $responseArray;
    }
    
} // end of class userFunctions
?>
